/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/
#include "ai.h"

AI::AI(Player *pl) {
    this->pl = pl;
    change.start();
    mode = IDLE;

    target = rand() % 360;
    nextChange = rand() % 3000 + 3000;
    forceBreaks = 0;
    breaksDuration = 0;


}

void AI::update(QVector<Player *> *players, QVector<MapObject *> *mapObjects) {
    right = false;
    left = false;
    up = true;
    down = false;
    shoot = false;

    if (change.elapsed() > nextChange) {
        nextChange = rand() % 3000 + 1000;
        change.restart();
        target = rand() % 360;
    }


    radius = pl->getCollisonRadius();
    //if (radius > 5)radius = 5;
    bool col = false;
    bool hardCol = false;
    bool playerCol = false;

    for (int i = 1; i < 6; i++) {
        double absSpeed = pl->getSpeed().length()*100 * i;
        double rad = osg::DegreesToRadians(target);
        osg::Vec3 pointTarget = pl->getPosition() + osg::Vec3(cos(rad) * absSpeed, sin(rad) * absSpeed, 0);
        osg::Vec3 pointReal = pl->getPosition() + pl->getSpeed()*100*i;
        for (int i = 0; i < mapObjects->count(); i++) {
            MapObject* m = mapObjects->at(i);
            if (m->collision(pointTarget, radius)) {
                col = true;
            }
            if (m->collision(pointReal, radius)) {
                hardCol = true;
            }
        }

        for (int i = 0; i < players->count(); i++) {
            Player* pb = players->at(i);
            if (pl == pb) continue;

            if (pb->collision(pointReal, radius)) {
                playerCol = true;
            }
        }
    }

    if (hardCol || col || playerCol) {
        forceBreaks = 50;

    }
    if (col) {
        target = ((int) pl->getYRotation() + 360 + 180) % 360;
    }

    if (forceBreaks > 0) {
        down = true;
        up = false;
        forceBreaks--;
    }




    double dif = target - pl->getYRotation();
    if (dif < 0)dif += 360;
    if (dif > 5 && dif < 180) {
        left = true;
    }
    if (dif < 355 && dif > 180) {
        right = true;
    }

    // anti-stuck
    if (down){
        breaksDuration++;
        if(breaksDuration>300){
            down=false;
            up=true;
            right=true;
        }
    }else{
        breaksDuration=0;
    }


    for(int i =0;i<players->count()-1;i++){
        Player* p = players->at(i);
        // skip self
        if(p==pl)continue;
        osg::Vec3 points[9];
        int ptr=0;
        points[ptr++]= p->getPosition();
        points[ptr++]= points[0]+osg::Vec3(300,200,0);
        points[ptr++]= points[0]+osg::Vec3(-300,200,0);
        points[ptr++]= points[0]+osg::Vec3(300,-200,0);
        points[ptr++]= points[0]+osg::Vec3(-300,-200,0);
        points[ptr++]= points[0]+osg::Vec3(0,200,0);
        points[ptr++]= points[0]+osg::Vec3(0,-200,0);
        points[ptr++]= points[0]+osg::Vec3(300,0,0);
        points[ptr++]= points[0]+osg::Vec3(-300,0,0);

        for(int j=0;j<9;j++){
            double distance = pl->distance(points[j]);

            // too far
            if (distance > 150) continue;


            osg::Vec3 objPoint = points[j] - pl->getPosition();

            double ata = osg::RadiansToDegrees(atan2(objPoint.x(), objPoint.y()));
            ata = 360 + 90 - ata;
            while (ata > 360)ata -= 360;
            double angle = ata - pl->getYRotation();
            if (abs(angle) > 15) continue;
            shoot=true;
            break;
        }
        if(shoot)break;
    }
}
