/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#ifndef AI_H
#define AI_H
class AI;

#include "player.h"
#include "mapobject.h"

#include <QVector>
#include <QTime>

/** 
 * Artificial intelligence * 
 */
class AI {
public:
    
    /**
     * Constructor
     * @param pl Controlled player
     */
    AI(Player *pl);

    /**
     * update AI
     * @param players List of players
     * @param mapObjects list of map objects
     */
    void update(QVector<Player*>* players, QVector<MapObject*>* mapObjects);

    
    bool isRight() {
        return right;
    }

    bool isLeft() {
        return left;
    }

    bool isUp() {
        return up;
    }

    bool isDown() {
        return down;
    }

    bool isShoot() {
        return shoot;
    }

private:

    enum state {
        STOP, IDLE, AVOID
    };
    state mode;
    Player *pl;
    QTime change;
    int nextChange;
    double target;
    bool right;
    bool left;
    bool up;
    bool down;
    bool shoot;

    int forceBreaks;
    int breaksDuration;
    double radius;
};

#endif // AI_H
