/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#include "bonus.h"

Bonus::Bonus() {
    expire = 10000;
    owner = NULL;
    collisionRadius = 3;
    createCircle();
}

void Bonus::update(int elapsed) {
    GameObject::update(elapsed);
    rotateY(0.2 * elapsed);
    expire -= elapsed;
}

void Bonus::apply(Player* pl) {
    owner = pl;
}

void Bonus::createCircle() {
    osg::Geode* circleGeode = new osg::Geode;
    osg::Geometry* circleGeometry = new osg::Geometry;


    osg::Vec3Array* vertices = new osg::Vec3Array(80);
    osg::Vec3Array::iterator vitr = vertices->begin();

    double cosy, sinx, rad, deg;
    for (int i = 0; i < 60; i++) {
        deg = 6 * i + 180;
        rad = osg::DegreesToRadians(deg);
        cosy = cos(rad);
        sinx = sin(rad);

        if (i % 3 == 0)
            (vitr++)->set(5 * sinx, 5 * cosy, -10);
        if (i % 3 == 1) {
            (vitr++)->set(6 * sinx, 6 * cosy, -10);
            (vitr++)->set(6 * sinx, 6 * cosy, -10);
        }
        if (i % 3 == 2) {
            (vitr++)->set(5 * sinx, 5 * cosy, -10);
        }
    }
    circleGeometry->setVertexArray(vertices);

    osg::Vec4Array* colors = new osg::Vec4Array;
    colors->push_back(osg::Vec4(1.0f, 1.0f, 1.0f, 1.0f));

    circleGeometry->setColorArray(colors);
    circleGeometry->setColorBinding(osg::Geometry::BIND_OVERALL);

    osg::Vec3Array* normals = new osg::Vec3Array;
    normals->push_back(osg::Vec3(0.0f, 0.0f, 1.0f));
    circleGeometry->setNormalArray(normals);
    circleGeometry->setNormalBinding(osg::Geometry::BIND_OVERALL);

    osg::DrawArrays* circle = new osg::DrawArrays(osg::PrimitiveSet::LINES, 0, 80);

    osg::LineWidth* linewidth = new osg::LineWidth();
    linewidth->setWidth(2.0f);
    circleGeometry->getOrCreateStateSet()->setAttributeAndModes(linewidth, osg::StateAttribute::ON);

    circleGeometry->addPrimitiveSet(circle);
    circleGeode->addDrawable(circleGeometry);
    modelPos->addChild(circleGeode);
}

BonusHeal::BonusHeal() {
    model->addChild(ModelLoader::instance()->modelBonusHeal.get());
}

void BonusHeal::apply(Player *pl) {
    Bonus::apply(pl);
    expire = 5000;
}

void BonusHeal::update(int elapsed) {
    Bonus::update(elapsed);

    if (owner && !owner->isDead()) {
        owner->setHpFull();
    }
}

BonusAmmo::BonusAmmo() {
    model->addChild(ModelLoader::instance()->modelBonusAmmo.get());
}

void BonusAmmo::apply(Player *pl) {
    Bonus::apply(pl);
    expire = 10000;
}

void BonusAmmo::update(int elapsed) {
    Bonus::update(elapsed);
    if (owner && !owner->isDead()) {
        owner->setAmmoFull();
    }
}

BonusLaser::BonusLaser() {
    model->addChild(ModelLoader::instance()->modelBonusLaser.get());
}

void BonusLaser::apply(Player *pl) {
    Bonus::apply(pl);
    pl->setWeapon(new Laser);
    expire = 0;
}

void BonusLaser::update(int elapsed) {
    Bonus::update(elapsed);
}

