/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/
#include "controls.h"


Controls* Controls::pinstance = NULL;

Controls::Controls(QObject *parent) :
QObject(parent) {
    // clear all flags
    for (int i = 0; i < MAX_PLAYERS; i++) {
        right[i] = false;
        left[i] = false;
        up[i] = false;
        down[i] = false;
        shoot[i] = false;
    }
    camZoom = false;
    camUnzoom = false;
    camUp = false;
    camDown = false;
    camSwitch = false;

    //preset bindings
    binds.insert(Qt::Key_W, &(up[0]));
    binds.insert(Qt::Key_S, &(down[0]));
    binds.insert(Qt::Key_A, &(left[0]));
    binds.insert(Qt::Key_D, &(right[0]));
    binds.insert(Qt::Key_Space, &(shoot[0]));

    binds.insert(Qt::Key_8, &(up[1]));
    binds.insert(Qt::Key_5, &(down[1]));
    binds.insert(Qt::Key_4, &(left[1]));
    binds.insert(Qt::Key_6, &(right[1]));
    binds.insert(Qt::Key_0, &(shoot[1]));

    binds.insert(Qt::Key_Up, &(up[2]));
    binds.insert(Qt::Key_Down, &(down[2]));
    binds.insert(Qt::Key_Left, &(left[2]));
    binds.insert(Qt::Key_Right, &(right[2]));
    binds.insert(Qt::Key_Control, &(shoot[2]));

    binds.insert(Qt::Key_I, &(up[3]));
    binds.insert(Qt::Key_K, &(down[3]));
    binds.insert(Qt::Key_J, &(left[3]));
    binds.insert(Qt::Key_L, &(right[3]));
    binds.insert(Qt::Key_AltGr, &(shoot[3]));

    binds.insert(Qt::Key_T, &(up[4]));
    binds.insert(Qt::Key_G, &(down[4]));
    binds.insert(Qt::Key_F, &(left[4]));
    binds.insert(Qt::Key_H, &(right[4]));


    binds.insert(Qt::Key_Plus, &(camZoom));
    binds.insert(Qt::Key_Minus, &(camUnzoom));
    binds.insert(Qt::Key_Asterisk, &(camSwitch));

}

Controls* Controls::instance() {
    if (pinstance == NULL) {
        pinstance = new Controls();
    }
    return pinstance;
}

bool Controls::press(int key) {
    if (binds.contains(key)) {
        *(binds[key]) = true;
        return true;
    } else {
        // not in player bindings
        switch (key) {
            case Qt::Key_F5:
                emit cheat1();
                break;
            case Qt::Key_F6:
                emit cheat2();
                break;
            case Qt::Key_F7:
                emit cheat3();
                break;
            case Qt::Key_F8:
                emit cheat4();
                break;
            default:
                return false;
        }
    }
    return true;
}

bool Controls::release(int key) {
    if (binds.contains(key)) {
        *(binds[key]) = false;
        return true;
    }
    return false;
}
