/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#ifndef GAMECONTROL_H
#define GAMECONTROL_H
class GameControl;
#include <QObject>
#include <QTimer>
#include <QTime>
#include <QLabel>
#include <QMessageBox>

#include "gamescene.h"
#include "controls.h"
#include "../settings.h"

#include "gameobject.h"
#include "mapobject.h"
#include "maps.h"

#include "gamecamera.h"
#include "player.h"
#include "projectile.h"
#include "bonus.h"


#include <osg/PositionAttitudeTransform>
#include <osg/MatrixTransform>
#include <osg/Matrixd>



#define OOR 20 // player out of reach

#define DEBUG 0
#define DEBUG_LOOP DEBUG&&1
#define DEBUG_PACK DEBUG&&1

/**
 * Class for controling game
 * Game loop and objects
 */
class GameControl : public QObject {
    Q_OBJECT
public:
    explicit GameControl(QObject *parent = 0);
    virtual ~GameControl();
    // sets scene

    void setGameScene(GameScene* gs) {
        this->gs = gs;
    }

    GameScene* getGameScene() {
        return gs;
    }

    // load game
    virtual void init();
    void init_shared();
    // start
    virtual void start();

    // add object to game
    void addMapObject(MapObject* obj);
protected:
    QLabel* lab;

    QTimer timer;
    QTime time;
    QTime loopTime;

    QTime bonusTime;

    int lps;
    bool started;
    int elapsed;
    int frames;
    QTime fps;

    GameScene* gs;

    Controls* cont;
    Settings* set;
    GameCamera* camera;

    Player* sputnik;
    QVector<Player*> players;
    QVector<Projectile*> projectiles;
    QVector<MapObject*> mapObjects;
    QVector<GameObject*> decoration;
    QVector<GameObject*> gravitySources;
    QVector<Bonus*> bonuses;

    virtual void updatePlayers(int elapsed);
    virtual void updateObjects(int elapsed);
    virtual void updateBonuses(int elapsed);
    virtual void updateCollisions(int elapsed);
    virtual void updateCamera(int elapsed);
    virtual void updateProjectiles(int elapsed);

    virtual void shoot(int player);
    void spawn(GameObject* p);

signals:
    void gameOver();

public slots:
    virtual void gameLoop();

    void slotCheat1();
    void slotCheat2();
    void slotCheat3();
    void slotCheat4();
};

#endif // GAMECONTROL_H
