/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#include "gamecontrolclient.h"

GameControlClient::GameControlClient(QObject *parent) :
    GameControl(parent) {
    con = set->getCurrentConnection();
    con->setParent(this);
    playerNumber = set->getPlayerNumber() - 1;
}

GameControlClient::~GameControlClient() {
}

void GameControlClient::init() {
    init_shared();

    // set camera mode
    camera->follow(players.at(playerNumber));
}

void GameControlClient::start() {
    GameControl::start();
    started = false;
    lastX = 0;
    lastY = 0;

    pingTimer.start();
    ping = 0;
    pong = true;
    if (DEBUG_LOOP)printf("gcc client started\n");
}

void GameControlClient::gameLoop() {
    if (DEBUG_LOOP)printf("gcc gameLoop Client\n");

    frames++;
    elapsed = loopTime.elapsed();
    loopTime.restart();
    lps = 1000 / (elapsed > 0 ? elapsed : 20);

    if (!started) {
        elapsed = 0;
    }

    if (pong && pingTimer.elapsed() > 1000) {
        pingSeq = rand();
        pong = false;
        IntPacket p(Packet::PING, pingSeq, ping);
        con->send(&p);
        pingTimer.restart();
    }


    // *********************
    // players
    if (DEBUG_LOOP)printf("read pack\n");
    readPackets();


    // *********************
    // players
    if (DEBUG_LOOP)printf("up pl\n");
    updatePlayers(elapsed);

    // *********************
    // bonuses
    if (DEBUG_LOOP)printf("up bon\n");
    updateBonuses(elapsed);


    // *********************
    // camera
    if (DEBUG_LOOP)printf("up cam\n");
    updateCamera(elapsed);

    // *********************
    // projectiles
    if (DEBUG_LOOP)printf("up pr\n");
    updateProjectiles(elapsed);

    // *********************
    // objects
    if (DEBUG_LOOP)printf("up ob\n");
    updateObjects(elapsed);

    // >>>>
    // debug output

    if(DEBUG){
        QString str;

        str += QString("Debug window:\n lps: %1\n")
                .arg(lps);
        str += QString(" in: %1 Bps\n out: %2 Bps\n")
                .arg(con->getRecievedps())
                .arg(con->getSentps());
        str += QString(" ping: %1\n")
                .arg(ping);
        lab->setText(str);
    }
    // <<<<
    if (DEBUG_LOOP)printf("up draw\n");
    gs->update();
}

void GameControlClient::readPackets() {
    Packet *p;
    while ((p = con->read())) {
        switch (p->type) {
        case Packet::GAME_START:
            if (DEBUG_PACK)printf("pack GAME_START \n");
            time.restart();
            started = true;
            loopTime.restart();
            break;
        case Packet::PLAYER_POSITION:
            if (DEBUG_PACK)printf("pack PLAYER_POSITION \n");
            packetPlayerPosition((PositionPacket*) p);
            break;
        case Packet::SPAWN:
            if (DEBUG_PACK)printf("pack SPAWN \n");
            spawn((PositionPacket*) p);
            break;
        case Packet::SHOOT:
            if (DEBUG_PACK)printf("pack SHOOT \n");
            packetShoot((IntPacket*) p);
            break;
        case Packet::DISCONNECTED:
            if (DEBUG_PACK)printf("pack DISCONNECTED \n");
            packetDisconnected((IntPacket*) p);
            break;
        case Packet::KILL:
            if (DEBUG_PACK)printf("pack KILL \n");
            packetKill((IntPacket*) p);
            break;
        case Packet::DAMAGE:
            if (DEBUG_PACK)printf("pack DAMAGE \n");
            packetDamage((IntPacket*) p);
            break;
        case Packet::SCORE:
            if (DEBUG_PACK)printf("pack SCORE \n");
            packetScore((ScorePacket*) p);
            break;
        case Packet::PING_REPLY:
            packetPingReply((IntPacket*) p);
            break;
        case Packet::BONUS_SPAWN:
            packetBonusSpawn((BonusPacket*) p);
            break;
        case Packet::BONUS_REMOVE:
            packetBonusRemove((IntPacket*) p);
            break;
        default:
            break;
        }
        delete p;
    }
}

void GameControlClient::updatePlayers(int elapsed) {

    for (int i = 0; i < players.count(); i++) {
        bool up = true;
        Player *pl = players.at(i);

        if (pl->isDead()) {
            // dead
            pl->setSpeed(osg::Vec3(0, 0, 0));
        } else {
            if (i == playerNumber) {
                // alive
                // movements
                if (cont->isLeft(0)) {
                    pl->steerLeft(elapsed);
                }
                if (cont->isRight(0)) {
                    pl->steerRight(elapsed);
                }
                if (cont->isUp(0)) {
                    pl->thrust(elapsed);
                }
                if (cont->isDown(0)) {
                    pl->breaks(elapsed);
                }
                // shooting
                if (cont->isShoot(0) && players.at(i)->isWeaponReady()) {
                    IntPacket p(Packet::WANT_SHOOT, 0, i);
                    con->send(&p);
                }

                if (abs(pl->getX() - lastX) > OOR || abs(pl->getY() - lastY) > OOR) {
                    // player position not synced
                    //pl->setX(lastX);
                    //pl->setY(lastY);
                    //up = false;
                }
            }
        }



        // portrait update
        if (pl != sputnik) {
            int ammo = pl->getWeapon()->getAmmo();
            int ammoMax = pl->getWeapon()->getAmmoMax();
            gs->getPortrait(i)->setHP(pl->getHp100());
            gs->getPortrait(i)->setShield(pl->getShield100());
            gs->getPortrait(i)->setAmmo(ammo, ammoMax);
            gs->getPortrait(i)->setScore(pl->getScore());
        }


        // move and check map bounds
        if (up)pl->update(elapsed);
        if (pl->getX() > 150)pl->setX(-150);
        if (pl->getX()<-150)pl->setX(150);
        if (pl->getY() > 100)pl->setY(-100);
        if (pl->getY()<-100)pl->setY(100);


        // send packet about player position

        if (i == playerNumber && frames % 3 == 0) {
            PositionPacket p(Packet::PLAYER_POSITION, pl->getX() * D2I, pl->getY() * D2I, pl->getYRotation() * D2I,
                             pl->getSpeed().x() * D2I,
                             pl->getSpeed().y() * D2I,
                             i, 0, time.elapsed());
            con->sendFast(&p);
        }
    }
}

void GameControlClient::updateBonuses(int elapsed) {
    for (int i = 0; i < bonuses.count(); i++) {
        Bonus* bon = bonuses.at(i);
        bon->update(elapsed);
        if (bon->isExpired()) {
            gs->removeModel(bon->getModel());
            bonuses.remove(i);
            i--;
        }
    }
}

void GameControlClient::packetPlayerPosition(PositionPacket *p) {
    Player * pl = players.at(p->player);
    if (p->player != playerNumber) {
        pl->setX(p->x / D2I);
        pl->setY(p->y / D2I);
        pl->setYRotation(p->rot / D2I);
        pl->setSpeed(osg::Vec3(p->speedx / D2I, p->speedy / D2I, 0));
    } else {
        lastX = p->x / D2I;
        lastY = p->y / D2I;
    }
}

void GameControlClient::spawn(PositionPacket *p) {
    Player * pl = players.at(p->player);
    pl->reset();
    pl->setX(p->x / D2I);
    pl->setY(p->y / D2I);
    pl->setYRotation(p->rot / D2I);
    pl->setSpeed(osg::Vec3(p->speedx / D2I, p->speedy / D2I, 0));
    if (p->player == playerNumber) {
        lastX = p->x / D2I;
        lastY = p->y / D2I;
    }
}

void GameControlClient::packetShoot(IntPacket *p) {
    printf("new projectile\n");
    shoot(p->player);
}

void GameControlClient::packetDisconnected(IntPacket *p) {
    QMessageBox msgBox;
    msgBox.move(gs->width() / 2 - 100, gs->height() / 2 - 50);
    msgBox.resize(200, 100);
    msgBox.setText(Texts::get(QString("CONNECTION_LOST")));
    msgBox.setStandardButtons(QMessageBox::Ok);
    gs->addWidget(&msgBox);
    Launcher::instance()->ignoreControls=true;
    msgBox.exec();
    Launcher::instance()->escape();
    Launcher::instance()->ignoreControls=false;
}

void GameControlClient::packetKill(IntPacket *p) {
    if (p->data == Packet::PROJECTILE) {
        printf("packet kill - proj\n");
        for (int i = 0; i < projectiles.count(); i++) {
            if (projectiles.at(i)->id == p->player) {
                projectiles.at(i)->setExpired();
                break;
            }
        }
    } else {
        printf("packet kill - player\n");
    }
    printf("packet kill end\n");
}

void GameControlClient::packetDamage(IntPacket *p) {
    printf("packet damage\n");
    players.at(p->player)->damage(p->data);
    if (players.at(p->player)->getHp() == 0) {
    }
    printf("packet damage end\n");
}

void GameControlClient::packetScore(ScorePacket *p) {
    printf("packet score\n");
    int i = p->player;
    Player *pl = players.at(i);
    pl->setHp(p->hp);
    pl->setShield(p->sp);
    pl->setScore(p->score);
    pl->setAmmo(p->ammo);
}

void GameControlClient::packetPingReply(IntPacket *p) {
    if (p->data == pingSeq) {
        ping = pingTimer.elapsed();
        pong = true;
    }
}

void GameControlClient::packetBonusSpawn(BonusPacket *p) {
    Bonus* newBonus = NULL;
    switch (p->bonusType) {
    case 0:
        newBonus = new BonusHeal;
        break;
    case 1:
        newBonus = new BonusAmmo;
        break;
    case 2:
        newBonus = new BonusLaser;
        break;
    }
    newBonus->setX(p->x);
    newBonus->setY(p->y);

    gs->addModel(newBonus->getModel());
    bonuses.append(newBonus);
}

void GameControlClient::packetBonusRemove(IntPacket *p){
    for(int i =0;i<bonuses.count();i++){
        Bonus* b = bonuses.at(i);
        if(b->getX()==p->data && b->getY()== p->player){
            gs->removeModel(b->getModel());
            break;
        }
    }
}
