/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/
#include "gamecontrolhost.h"

GameControlHost::GameControlHost(QObject* parent) :
GameControl(parent) {
    //printf("GameControlHost constructor\n");
    con = (HostConnection*) set->getCurrentConnection();
    con->setParent(this);
    //printf("GameControlHost constructor\n end");
}

GameControlHost::~GameControlHost() {
}

void GameControlHost::init() {
    init_shared();
    // set camera mode
    camera->follow(players.at(0));

}

void GameControlHost::start() {
    GameControl::start();
    started = false;
}

void GameControlHost::gameLoop() {
    frames++;
    elapsed = loopTime.elapsed();
    loopTime.restart();

    lps = 1000 / (elapsed > 0 ? elapsed : 20);

    if (!started) {
        elapsed = 0;

        if (time.elapsed() > 3000) {
            Packet p(Packet::GAME_START);
            con->send(&p);
            time.restart();
            started = true;

            for (int i = 0; i < players.count(); i++) {
                Player* pl = players.at(i);
                PositionPacket p(Packet::SPAWN, pl->getX() * D2I, pl->getY() * D2I, pl->getYRotation() * D2I,
                        0, 0,
                        i, 0, time.elapsed());
                con->sendFast(&p);
            }
        }
    }

    // *********************
    // players
    if (DEBUG_LOOP)printf("read pack\n");
    readPackets();

    // *********************
    // players
    if (DEBUG_LOOP)printf("up pl\n");
    updatePlayers(elapsed);

    // *********************
    // bonuses
    if (DEBUG_LOOP)printf("up bon\n");
    updateBonuses(elapsed);

    // *********************
    // camera
    if (DEBUG_LOOP)printf("up cam\n");
    updateCamera(elapsed);

    // *********************
    // projectiles
    if (DEBUG_LOOP)printf("up pr\n");
    updateProjectiles(elapsed);

    // *********************
    // objects
    if (DEBUG_LOOP)printf("up ob\n");
    updateCollisions(elapsed);

    // *********************
    // collisions
    if (DEBUG_LOOP)printf("up col\n");
    updateObjects(elapsed);

    // >>>>
    // debug output
    if (DEBUG) {
        QString str;
        str += QString("Debug window:\n lps: %1\n")
                .arg(lps);
        str += QString(" in: %1 B/s\n out: %2 B/s\n")
                .arg(con->getRecievedps())
                .arg(con->getSentps());
        lab->setText(str);
        // <<<<
    }

    gs->update();

}

void GameControlHost::readPackets() {
    Packet *p;
    while ((p = con->read())) {
        switch (p->type) {
            case Packet::PLAYER_POSITION:
                packetPlayerPosition((PositionPacket*) p);
                break;
            case Packet::WANT_SHOOT:
                packetWantShoot((IntPacket*) p);
            case Packet::PING:
                packetPing((IntPacket*) p);
                break;
            default:
                break;
        }
        delete p;
    }
}

void GameControlHost::updatePlayers(int elapsed) {
    for (int i = 0; i < players.count(); i++) {
        Player *pl = players.at(i);

        if (pl->isDead()) {
            // dead
            pl->setSpeed(osg::Vec3(0, 0, 0));
            if (pl->getTimer() > 2000) {
                spawn(pl);
                pl->reset();
            }
        } else {
            if (i == 0) {
                // alive
                // movements
                if (cont->isLeft(i)) {
                    pl->steerLeft(elapsed);
                }
                if (cont->isRight(i)) {
                    pl->steerRight(elapsed);
                }
                if (cont->isUp(i)) {
                    pl->thrust(elapsed);
                }
                if (cont->isDown(i)) {
                    pl->breaks(elapsed);
                }
                // shooting
                if (cont->isShoot(i) && players.at(i)->isWeaponReady()) {
                    shoot(i);
                    IntPacket p(Packet::SHOOT, 0, i);
                    con->send(&p);
                }
            }
            if (pl == sputnik) {
                // alive
                pl->ai->update(&players, &mapObjects);
                // movements
                if (pl->ai->isLeft()) {
                    pl->steerLeft(elapsed);
                }
                if (pl->ai->isRight()) {
                    pl->steerRight(elapsed);
                }
                if (pl->ai->isUp()) {
                    pl->thrust(elapsed);
                }
                if (pl->ai->isDown()) {
                    pl->breaks(elapsed);
                }
            }
        }

        // move and check map bounds
        pl->update(elapsed);
        if (pl->getX() > 150) pl->setX(-150);
        if (pl->getX() <-150) pl->setX(150);
        if (pl->getY() > 100) pl->setY(-100);
        if (pl->getY() <-100) pl->setY(100);


        // portrait update
        if (pl != sputnik) {
            int ammo = pl->getWeapon()->getAmmo();
            int ammoMax = pl->getWeapon()->getAmmoMax();
            gs->getPortrait(i)->setHP(pl->getHp100());
            gs->getPortrait(i)->setShield(pl->getShield100());
            gs->getPortrait(i)->setAmmo(ammo, ammoMax);
            gs->getPortrait(i)->setScore(pl->getScore());
        }


        // send packet about player position
        if (frames % 3 == 0) {
            PositionPacket p(Packet::PLAYER_POSITION, pl->getX() * D2I, pl->getY() * D2I, pl->getYRotation() * D2I,
                    pl->getSpeed().x() * D2I,
                    pl->getSpeed().y() * D2I, i, 0,
                    time.elapsed());
            con->sendFast(&p);
        }
        if (frames % 25 == 0) {
            ScorePacket p(i, pl->getHp(), pl->getShield(), pl->getWeapon()->getAmmo(), pl->getScore());
            con->send(&p);
        }
    }
}

void GameControlHost::updateBonuses(int elapsed) {
    if (bonusTime.elapsed() > 5000) {
        bonusTime.restart();
        // spawn bonus with chance
        if (rand() % 100 > 50) { // 50%
            int type = rand() % 2;

            Bonus* newBonus = NULL;
            switch (type) {
                case 0:
                    newBonus = new BonusHeal;
                    break;
                case 1:
                    newBonus = new BonusAmmo;
                    break;
            }
            GameControl::spawn(newBonus);
            gs->addModel(newBonus->getModel());
            bonuses.append(newBonus);

            BonusPacket p(type, newBonus->getX(), newBonus->getY());
            con->send(&p);
        }
    }
    for (int i = 0; i < bonuses.count(); i++) {
        Bonus* bon = bonuses.at(i);
        bon->update(elapsed);
        if (bon->isExpired()) {
            gs->removeModel(bon->getModel());
            bonuses.remove(i);
            i--;
        }
    }
}

void GameControlHost::updateCollisions(int elapsed) {
    // players x players
    for (int a = 0; a < players.count() - 1; a++) {
        Player* ap = players.at(a);
        for (int b = 0; b < players.count(); b++) {
            if (a == b) continue;
            Player* bp = players.at(b);
            if (ap->collision(bp)) {
                // player collision, kill them both!
                ap->damage(99999);
                bp->damage(99999);
                ap->scoreUp(-1);
                bp->scoreUp(-1);
                if (bp == sputnik)
                    ap->scoreUp(-1);

                //CollisionPacket p(CollisionPacket::PvP,ap->id,bp->id,time.elapsed());
                IntPacket p1(Packet::DAMAGE, 99999, a);
                IntPacket p2(Packet::DAMAGE, 99999, b);
                IntPacket p3(Packet::KILL, Packet::PLAYER, a);
                IntPacket p4(Packet::KILL, Packet::PLAYER, b);
                con->send(&p1);
                con->send(&p2);
                con->send(&p3);
                con->send(&p4);
            }


            if (ap->isDead() && ap->collisionHard(bp)) {
                ap->setSpeed(osg::Vec3(0, 0, 0));
            }
        }
    }

    // projectiles x players
    for (int a = 0; a < projectiles.count(); a++) {
        Projectile* ap = projectiles.at(a);
        for (int b = 0; b < players.count(); b++) {
            Player* bp = players.at(b);
            if (ap->collision(bp)) {
                if (ap->getOwner() == bp) continue;
                // player took hit
                bp->damage(ap->getDamage());
                ap->setExpired();

                //CollisionPacket p(CollisionPacket::PvB,bp->id,ap->id,time.elapsed());
                IntPacket p1(Packet::DAMAGE, ap->getDamage(), b);
                IntPacket p2(Packet::KILL, Packet::PROJECTILE, ap->id);
                con->send(&p1);
                con->send(&p2);

                if (bp->isDead()) {
                    // and died
                    Player* pl = dynamic_cast<Player*> (ap->getOwner());
                    pl->scoreUp(1);

                    if (bp == sputnik) {
                        pl->scoreUp(-3);
                    }


                }

            }
        }
    }

    // projectiles x mapobjects
    for (int a = 0; a < projectiles.count(); a++) {
        Projectile* ap = projectiles.at(a);
        for (int b = 0; b < mapObjects.count(); b++) {
            MapObject* bp = mapObjects.at(b);
            if (ap->collision(bp)) {
                // shooting rocks, remove bullet
                ap->setExpired();

                //CollisionPacket p(CollisionPacket::EvB,bp->id,ap->id,time.elapsed());
                IntPacket p1(Packet::KILL, Packet::PROJECTILE, ap->id);
                con->send(&p1);
            }
        }
    }

    // players x mapobjects
    for (int a = 0; a < players.count(); a++) {
        Player* ap = players.at(a);
        for (int b = 0; b < mapObjects.count(); b++) {
            MapObject* bp = mapObjects.at(b);
            if (ap->collision(bp)) {
                // that asteroid didnt move away =( Kill player
                ap->damage(99999);
                ap->scoreUp(-1);

                //CollisionPacket p(CollisionPacket::PvE,ap->id,bp->id,time.elapsed());
                IntPacket p1(Packet::DAMAGE, 99999, a);
                IntPacket p2(Packet::KILL, Packet::PLAYER, a);
                con->send(&p1);
                con->send(&p2);
            }
        }
    }

    // players x bonus
    for (int a = 0; a < players.count() - 1; a++) {
        Player* ap = players.at(a);
        for (int b = 0; b < bonuses.count(); b++) {
            Bonus* bp = bonuses.at(b);
            if (ap->collision(bp)) {
                // player picks up a bonus
                bp->apply(ap);
                gs->removeModel(bp->getModel());
                IntPacket p(Packet::BONUS_REMOVE,bp->getX(),bp->getY());
                con->send(&p);
            }
        }
    }

}

void GameControlHost::packetPlayerPosition(PositionPacket *p) {
    Player * pl = players.at(p->player);
    pl->setYRotation(p->rot / D2I);

    int x = p->x / D2I;
    int y = p->y / D2I;
    pl->setSpeed(osg::Vec3(p->speedx / D2I, p->speedy / D2I, 0));
    if (abs(pl->getX() - x) > OOR || abs(pl->getY() - y) > OOR) {
        // player position not synced, ignore
        printf("%d not synced\n", p->player);
        return;
    }
    pl->setX(p->x / D2I);
    pl->setY(p->y / D2I);
}

void GameControlHost::packetWantShoot(IntPacket *p) {
    Player *pl = players.at(p->player);
    if (pl->isWeaponReady()) {
        IntPacket pp(Packet::SHOOT, 0, p->player);
        con->send(&pp);
        shoot(p->player);
    } else {
        // @todo shoot later?
    }
}

void GameControlHost::packetPing(IntPacket *p) {
    IntPacket pp(Packet::PING_REPLY, p->data, 0);
    con->send(&pp);
}

void GameControlHost::spawn(Player *pl) {
    GameControl::spawn(pl);
    // send packet about player spawn
    PositionPacket p(Packet::SPAWN, pl->getX() * D2I, pl->getY() * D2I, pl->getYRotation() * D2I,
            0, 0,
            players.indexOf(pl), 0, time.elapsed());
    con->sendFast(&p);
}


























