/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#include "gameobject.h"

uint GameObject::nextID = 0;

GameObject::GameObject() {
    // default values
    id = nextID;
    nextID++;

    collisionImmune = false;
    gravityImmune = false;

    x = 0;
    y = 0;
    z = 0;
    speedDouble = 0.001;
    yrotation = 0;
    maxSpeed = 0.06;
    minSpeed = 0;
    speed = osg::Vec3(0.0, 0.0, 0.0);
    model = new osg::PositionAttitudeTransform();
    modelPos = new osg::PositionAttitudeTransform();

    model->addChild(modelPos);
}

GameObject::~GameObject() {

}

void GameObject::setX(double x) {
    this->x = x;
}

void GameObject::setY(double y) {
    this->y = y;
}

void GameObject::setZ(double z) {
    this->z = z;
}

void GameObject::setPosition(osg::Vec3 pos) {
    x = pos.x();
    y = pos.y();
    z = pos.z();
}

void GameObject::setYRotation(double rot) {
    this->yrotation = rot;
}

void GameObject::setSpeed(double sp) {
    if (sp > maxSpeed)sp = maxSpeed;
    double rad = osg::DegreesToRadians(yrotation);
    speed = osg::Vec3(sp * cos(rad), sp * sin(rad), 0);
}

void GameObject::setSpeed(osg::Vec3 sp) {
    this->speed = sp;
}

void GameObject::rotateY(double rot) {
    yrotation += rot;
    if (yrotation > 360)yrotation -= 360;
    if (yrotation < 0)yrotation += 360;
}

void GameObject::speedUp(osg::Vec3 sp) {
    speed += sp;
    if (speed.length() > maxSpeed) {
        speed.normalize();
        speed *= maxSpeed;
    }
}

void GameObject::speedUp(double sp) {
    double rad = osg::DegreesToRadians(yrotation);
    speedUp(osg::Vec3(cos(rad) * sp, sin(rad) * sp, 0));
}

void GameObject::update(int elapsed) {
    double rad = osg::DegreesToRadians(yrotation);
    x += elapsed * speed.x();
    y += elapsed * speed.y();

    model->setPosition(osg::Vec3(x, y, z));
    model->setAttitude(osg::Quat(rad, osg::Vec3(0, 0, 1)));
    modelPos->setAttitude(osg::Quat(-rad, osg::Vec3(0, 0, 1)));
}

bool GameObject::collision(GameObject* obj) {
    if (isCollisionImmune() || obj->isCollisionImmune()) return false;
    return collisionHard(obj);
}

bool GameObject::collision(osg::Vec3 pos, double rad) {
    if (isCollisionImmune()) return false;
    return collisionHard(pos, rad);
}

bool GameObject::collisionHard(GameObject* obj) {
    return distance(obj) < collisionRadius + obj->getCollisonRadius();
}

bool GameObject::collisionHard(osg::Vec3 pos, double rad) {
    return distance(pos) < collisionRadius + rad;
}

void GameObject::setCollisionRadius(double r) {
    collisionRadius = r;
}

double GameObject::distance(GameObject *obj) {
    return distance(obj->getPosition());
}

double GameObject::distance(osg::Vec3 pos) {
    double x2 = (x - pos.x())*(x - pos.x());
    double y2 = (y - pos.y())*(y - pos.y());
    double z2 = (z - pos.z())*(z - pos.z());
    return sqrt(x2 + y2 + z2);
}

double GameObject::angleTo(GameObject *obj) {
    osg::Vec3 objPoint = obj->getPosition() - getPosition();
    return osg::RadiansToDegrees(atan2(objPoint.x(), objPoint.y())) - yrotation;
}
