#ifndef GAMEOBJECT_H
#define GAMEOBJECT_H

#include <osg/PositionAttitudeTransform>
/*****************************************************************************
 * ISP/IBP 2011/2012
 * Author: Michal Toth (xtothm00@stud.fit.vutbr.cz)
 *
 *****************************************************************************/

#include <osgDB/ReadFile>
#include <osg/Node>

#include <QString>

#include "../modelloader.h"

/**
 * Abstract class for Game object
 * -position and speed
 * -flags for immunities
 * -collision calculation
 */
class GameObject {
public:
    static uint nextID;
    int id;

    GameObject();
    virtual ~GameObject();

    // positioning
    void setX(double x);
    void setY(double y);
    void setZ(double z);
    void setYRotation(double rot);
    void setSpeed(double sp);
    void setSpeed(osg::Vec3 sp);
    void setPosition(osg::Vec3 pos);
    void rotateY(double rot);
    void speedUp(osg::Vec3 sp);
    void speedUp(double sp);


    // getters

    double getX() {
        return x;
    }

    double getY() {
        return y;
    }

    double getZ() {
        return z;
    }

    osg::Vec3 getPosition() {
        return osg::Vec3(x, y, z);
    }

    double getYRotation() {
        return yrotation;
    }

    osg::Vec3 getSpeed() {
        return speed;
    }

    // immunities

    virtual double getCollisonRadius() {
        return collisionRadius;
    }

    void setCollisionImmune(bool b) {
        collisionImmune = b;
    }

    virtual bool isCollisionImmune() {
        return collisionImmune;
    }

    bool isGravityImmune() {
        return gravityImmune;
    }

    // collision with other object
    bool collision(GameObject* obj);
    bool collision(osg::Vec3 obj,double rad);
    // callision ignoring immunity
    bool collisionHard(GameObject* obj);
    bool collisionHard(osg::Vec3 pos,double rad);

    void setCollisionRadius(double r);

    double distance(GameObject* obj);
    double distance(osg::Vec3 pos);
    double angleTo(GameObject* obj);

    virtual void update(int elapsed);

    osg::Node* getModel() {
        return model.get();
    }

protected:
    double x, y, z;
    double yrotation;
    double speedDouble, maxSpeed, minSpeed;

    osg::Vec3 speed;

    bool collisionImmune;
    bool gravityImmune;
    double collisionRadius;


    osg::ref_ptr<osg::PositionAttitudeTransform> model;
    osg::ref_ptr<osg::PositionAttitudeTransform> modelPos;
};

#endif // GAMEOBJECT_H
