/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#include "gamescene.h"

GameScene::GameScene(QObject *parent) :
QOSGScene(parent) {
    std::cout << "GameScene contructor\n";
    visible = true;
    scene = new osg::Group;

    setSceneData(scene);
    makeGrid();
    update();
    std::cout << "GameScene contructor end\n";


    QHBoxLayout* lay = new QHBoxLayout();
    for (int i = 0; i < 4; i++) {
        QLabel* l = new QLabel;
        l->hide();
        l->setObjectName("portrait");
        PlayerPortrait* p = new PlayerPortrait(l);
        l->resize(p->size());
        l->setMaximumSize(p->size());
        portraits[i] = p;
        lay->addWidget(l);
    }

    portraitsLabel = new QLabel;
    portraitsLabel->setAttribute(Qt::WA_NoBackground);
    portraitsLabel->setGeometry(100, height() - 150, width() - 200, 160);
    portraitsLabel->setLayout(lay);
    addToHUD(portraitsLabel);
    if (!Settings::instance()->plUi2D) {
        portraitsLabel->hide();
    }

    connect(this, SIGNAL(resized()), this, SLOT(resized()));
}

GameScene::~GameScene() {

}

void GameScene::makeGrid() {
    // @todo generovat velikost z mapy. ne na pevno
    grid = new osg::Geode();
    osg::Geometry* geom = new osg::Geometry();

    osg::Vec3Array* vertices = new osg::Vec3Array(59 * 2 + 39 * 2);
    osg::Vec3Array::iterator vitr = vertices->begin();
    for (int i = -145; i < 150; i += 5) {
        (vitr++)->set(i, 100, -5);
        (vitr++)->set(i, -100, -5);
    }
    for (int i = -95; i < 100; i += 5) {
        (vitr++)->set(150, i, -5);
        (vitr++)->set(-150, i, -5);
    }
    geom->setVertexArray(vertices);
    osg::Vec4Array* colors = new osg::Vec4Array;
    colors->push_back(osg::Vec4(0.0f, 0.2f, 0.0f, 0.2f));
    geom->setColorArray(colors);
    geom->setColorBinding(osg::Geometry::BIND_OVERALL);

    osg::Vec3Array* normals = new osg::Vec3Array;
    normals->push_back(osg::Vec3(0.0f, 0.0f, 1.0f));
    geom->setNormalArray(normals);
    geom->setNormalBinding(osg::Geometry::BIND_OVERALL);

    geom->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::LINES, 0, 59 * 2 + 39 * 2));
    grid->addDrawable(geom);



    geom = new osg::Geometry();
    vertices = new osg::Vec3Array(4);
    vitr = vertices->begin();
    (vitr++)->set(-150, 100, -5);
    (vitr++)->set(150, 100, -5);
    (vitr++)->set(150, -100, -5);
    (vitr++)->set(-150, -100, -5);
    geom->setVertexArray(vertices);
    colors = new osg::Vec4Array;
    colors->push_back(osg::Vec4(0.5f, 0.0f, 0.0f, 0.5f));
    geom->setColorArray(colors);
    geom->setColorBinding(osg::Geometry::BIND_OVERALL);

    normals = new osg::Vec3Array;
    //normals->push_back(osg::Vec3(0.0f,-1.0f,0.0f));
    normals->push_back(osg::Vec3(0.0f, 0.0f, 1.0f));
    geom->setNormalArray(normals);
    geom->setNormalBinding(osg::Geometry::BIND_OVERALL);

    osg::LineWidth* linewidth = new osg::LineWidth();
    linewidth->setWidth(10.0f);
    geom->getOrCreateStateSet()->setAttributeAndModes(linewidth,
            osg::StateAttribute::ON);


    geom->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::LINE_LOOP, 0, 4));
    grid->addDrawable(geom);
}

void GameScene::setGridVisible(bool t) {
    if (t) {
        scene->addChild(grid);
    } else {
        scene->removeChild(grid);
    }
}

void GameScene::resized() {
    portraitsLabel->setGeometry(100, height() - 200, width() - 200, 160);
}

void GameScene::switchHUDVisible() {
    visible = !visible;

    foreach(QWidget* w, HUD) {
        w->setVisible(visible);
    }
}

void GameScene::addToHUD(QWidget *widget) {
    addWidget(widget);
    HUD.append(widget);
}
