/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#include "player.h"
#include <osg/ShapeDrawable>
#include <osg/Node>

Player* Player::newPlayer(int type, GameScene *gs) {
    switch (type) {
        case WRAITH:
            return new PlayerWraith(gs);
        case RAIDER:
            return new PlayerRaider(gs);
    }
    return NULL;
}

Player::Player() : GameObject() {
    explosion = NULL;
    wep = NULL;

    circleHP = NULL;
    circleSP = NULL;
    circleAmmo = NULL;
    circleGeometryHP = NULL;
    circleGeometrySP = NULL;
    circleGeometryAmmo = NULL;
    circleGeodeAmmo = NULL;

    // @todo
    // load points per type
    weaponPoint[0] = osg::Vec3(0, 0, 0);
    weaponPoint[1] = osg::Vec3(2, -3, 0);
    weaponPoint[2] = osg::Vec3(2, 3, 0);

    // default values
    score = 0;
    collisionRadius = 7;
    collisionRadiusShield = 10;
    shieldMax = 500;
    hpMax = 1500;

    shieldCDmax = 2000;
    shieldRate = 1;

    steerSpeed = 0.2;
    acceleration = 0.0002;
    maxSpeed = 0.045;

    if (Settings::instance()->modeDmg) {
        hpMax = 1;
        shieldMax = 1;
    }

    hpRadius = 7;

    // create shield
    osg::ShapeDrawable* sp = new osg::ShapeDrawable(new osg::Sphere(osg::Vec3(0, 0, 0), 10));
    sp->setColor(osg::Vec4(0, 0.8, 0.8, 0.25));
    sp->getOrCreateStateSet()->setMode(GL_BLEND, osg::StateAttribute::ON);


    shieldModel = new osg::Geode;
    shieldModel->addDrawable(sp);
    shieldUp = false;


    reset();

    createHPCircle();
    createSPCircle();
    createAmmoCircle();

    ai = new AI(this);
    ignoreControls = false;

}

PlayerWraith::PlayerWraith(GameScene *gs) : Player() {
    model->addChild(ModelLoader::instance()->modelWraith.get());
    EngineSmoke2* es = new EngineSmoke2;


    osg::PositionAttitudeTransform* tx = new osg::PositionAttitudeTransform;
    tx->addChild(es);
    tx->setPosition(osg::Vec3(-6, 0, 1));
    tx->setAttitude(osg::Quat(osg::DegreesToRadians(90.0), osg::Vec3(0, -1, 0)));

    model->addChild(tx);
    gs->addModel(es->geode);
    gs->addModel(es->psu);
}

PlayerRaider::PlayerRaider(GameScene *gs) : Player() {
    model->addChild(ModelLoader::instance()->modelRaider.get());

    EngineSmoke3* es = new EngineSmoke3;
    EngineSmoke3* es2 = new EngineSmoke3;


    osg::PositionAttitudeTransform* tx = new osg::PositionAttitudeTransform;
    tx->addChild(es);
    tx->setPosition(osg::Vec3(-6, -2, 1));
    tx->setAttitude(osg::Quat(osg::DegreesToRadians(90.0), osg::Vec3(0, -1, 0)));

    osg::PositionAttitudeTransform* tx2 = new osg::PositionAttitudeTransform;
    tx2->addChild(es2);
    tx2->setPosition(osg::Vec3(-6, 2, 1));
    tx2->setAttitude(osg::Quat(osg::DegreesToRadians(90.0), osg::Vec3(0, -1, 0)));


    model->addChild(tx);
    model->addChild(tx2);
    gs->addModel(es->geode);
    gs->addModel(es->psu);

    gs->addModel(es2->geode);
    gs->addModel(es2->psu);
}

PlayerSputnik::PlayerSputnik() : Player() {
    model->addChild(ModelLoader::instance()->modelSputnik.get());
    collisionRadius = 3;
    shieldMax = 0;
    shield = shieldMax;
    hpMax = 1;
    hp = hpMax;
    maxSpeed = 0.02;
}

void PlayerSputnik::update(int elapsed) {
    Player::update(elapsed);


    wep->setAmmo(0);
    circleGeometryAmmo->removePrimitiveSet(0);
    int ammo = 0;
    circleAmmo->setCount(ammo);
    circleGeometryAmmo->addPrimitiveSet(circleAmmo);
}

void Player::reset() {
    // default weapon
    setWeapon(new Chaingun);

    shield = shieldMax;
    shieldCD = shieldCDmax;
    hp = hpMax;
    timer = 0;
    shieldTimer = -1;
    setSpeed(osg::Vec3(0, 0, 0));
    setShieldDown();

    if (explosion) {
        model->removeChild(explosion);
        explosion = NULL;
    }
}

void Player::update(int elapsed) {
    GameObject::update(elapsed);

    if (wep->type != Weapon::CHAINGUN && wep->getAmmo() == 0) {
        setWeapon(new Chaingun);
    }


    if (!isDead()) {
        wep->update(elapsed);
        if (Settings::instance()->modeAmmo) {
            setAmmoFull();
        }
    } else {
        //speed = osg::Vec3(0,0,0);
    }
    timer += elapsed;
    if (shieldUp) {
        shieldTimer -= elapsed;
        if (shieldTimer <= 0)
            setShieldDown();
    }

    if (shieldCD <= 0 && hp > 0) {
        shield += shieldRate*elapsed;
        if (shield > shieldMax)shield = shieldMax;
    } else {
        shieldCD -= elapsed;
    }

    circleGeometryHP->removePrimitiveSet(0);
    circleHP->setCount((getHp100()*2) / 5);
    circleGeometryHP->addPrimitiveSet(circleHP);

    circleGeometrySP->removePrimitiveSet(0);
    circleSP->setCount((getShield100()*2) / 5);
    circleGeometrySP->addPrimitiveSet(circleSP);

    circleGeometryAmmo->removePrimitiveSet(0);
    int ammo = (wep->getAmmo100()*2) / 5;
    if (ammo < 0)ammo = 0;
    circleAmmo->setCount(ammo);
    circleGeometryAmmo->addPrimitiveSet(circleAmmo);
}

void Player::updatePos(int elapsed) {
    GameObject::update(elapsed);

}

void Player::setShieldUp() {

    if (!shieldUp)
        model->addChild(shieldModel.get());
    shieldTimer = 500;
    shieldUp = true;
}

void Player::setShieldDown() {
    model->removeChild(shieldModel.get());
    shieldUp = false;
}

void Player::createHPCircle() {
    osg::Geode* circleGeodeHP = new osg::Geode;
    circleGeometryHP = new osg::Geometry;


    osg::Vec3Array* vertices = new osg::Vec3Array(40);
    osg::Vec3Array::iterator vitr = vertices->begin();

    double cosy, sinx, rad, deg;
    for (int i = 0; i < 40; i++) {
        deg = 9 * i + 180;
        rad = osg::DegreesToRadians(deg);
        cosy = cos(rad);
        sinx = sin(rad);

        (vitr++)->set(hpRadius * sinx, hpRadius * cosy, -10);
    }
    circleGeometryHP->setVertexArray(vertices);

    osg::Vec4Array* colors = new osg::Vec4Array;
    colors->push_back(osg::Vec4(0.0f, 1.0f, 0.0f, 1.0f));

    circleGeometryHP->setColorArray(colors);
    circleGeometryHP->setColorBinding(osg::Geometry::BIND_OVERALL);

    osg::Vec3Array* normals = new osg::Vec3Array;
    normals->push_back(osg::Vec3(0.0f, 0.0f, 1.0f));
    circleGeometryHP->setNormalArray(normals);
    circleGeometryHP->setNormalBinding(osg::Geometry::BIND_OVERALL);

    circleHP = new osg::DrawArrays(osg::PrimitiveSet::LINES, 0, 40);

    osg::LineWidth* linewidth = new osg::LineWidth();
    linewidth->setWidth(2.0f);
    circleGeometryHP->getOrCreateStateSet()->setAttributeAndModes(linewidth, osg::StateAttribute::ON);

    circleGeometryHP->addPrimitiveSet(circleHP);
    circleGeodeHP->addDrawable(circleGeometryHP);

    if (Settings::instance()->plUi3D) {
        modelPos->addChild(circleGeodeHP);
    }
}

void Player::createSPCircle() {
    osg::Geode* circleGeodeSp = new osg::Geode;
    circleGeometrySP = new osg::Geometry;


    osg::Vec3Array* vertices = new osg::Vec3Array(40);
    osg::Vec3Array::iterator vitr = vertices->begin();

    double cosy, sinx, rad, deg;
    for (int i = 0; i < 40; i++) {
        deg = 9 * i + 180;
        rad = osg::DegreesToRadians(deg);
        cosy = cos(rad);
        sinx = sin(rad);
        (vitr++)->set((hpRadius + 0.5) * sinx, (hpRadius + 0.5) * cosy, -10);
    }
    circleGeometrySP->setVertexArray(vertices);

    osg::Vec4Array* colors = new osg::Vec4Array;
    colors->push_back(osg::Vec4(0.0f, 1.0f, 1.0f, 1.0f));

    circleGeometrySP->setColorArray(colors);
    circleGeometrySP->setColorBinding(osg::Geometry::BIND_OVERALL);

    osg::Vec3Array* normals = new osg::Vec3Array;
    normals->push_back(osg::Vec3(0.0f, 0.0f, 1.0f));
    circleGeometrySP->setNormalArray(normals);
    circleGeometrySP->setNormalBinding(osg::Geometry::BIND_OVERALL);

    circleSP = new osg::DrawArrays(osg::PrimitiveSet::LINES, 0, 40);

    osg::LineWidth* linewidth = new osg::LineWidth();
    linewidth->setWidth(2.0f);
    circleGeometrySP->getOrCreateStateSet()->setAttributeAndModes(linewidth, osg::StateAttribute::ON);

    circleGeometrySP->addPrimitiveSet(circleSP);
    circleGeodeSp->addDrawable(circleGeometrySP);
    if (Settings::instance()->plUi3D) {
        modelPos->addChild(circleGeodeSp);
    }
}

void Player::createAmmoCircle() {
    circleGeodeAmmo = new osg::Geode;
    circleGeometryAmmo = new osg::Geometry;

    osg::Vec3Array* vertices = new osg::Vec3Array(40);
    osg::Vec3Array::iterator vitr = vertices->begin();

    double cosy, sinx, rad, deg;
    for (int i = 0; i < 40; i++) {
        deg = 9 * i + 180;
        rad = osg::DegreesToRadians(deg);
        cosy = cos(rad);
        sinx = sin(rad);
        (vitr++)->set((hpRadius + 1) * sinx, (hpRadius + 1) * cosy, -10);
    }
    circleGeometryAmmo->setVertexArray(vertices);

    osg::Vec4Array* colors = new osg::Vec4Array;
    colors->push_back(osg::Vec4(1.0f, 1.0f, 0.0f, 1.0f));

    circleGeometryAmmo->setColorArray(colors);
    circleGeometryAmmo->setColorBinding(osg::Geometry::BIND_OVERALL);

    osg::Vec3Array* normals = new osg::Vec3Array;
    normals->push_back(osg::Vec3(0.0f, 0.0f, 1.0f));
    circleGeometryAmmo->setNormalArray(normals);
    circleGeometryAmmo->setNormalBinding(osg::Geometry::BIND_OVERALL);

    circleAmmo = new osg::DrawArrays(osg::PrimitiveSet::LINES, 0, 40);

    osg::LineWidth* linewidth = new osg::LineWidth();
    linewidth->setWidth(2.0f);
    circleGeometryAmmo->getOrCreateStateSet()->setAttributeAndModes(linewidth, osg::StateAttribute::ON);

    circleGeometryAmmo->addPrimitiveSet(circleAmmo);
    circleGeodeAmmo->addDrawable(circleGeometryAmmo);
    if (Settings::instance()->plUi3D) {
        modelPos->addChild(circleGeodeAmmo);
    }
}

bool Player::isWeaponReady() {
    return wep->isReady();
}

void Player::damage(int dmg) {
    if (hp <= 0)return;
    if (shield != 0) { // activate shield
        shieldCD = shieldCDmax;
        setShieldUp();
    }
    if (shield > dmg) { // all dmg eaten by shield
        shield -= dmg;
    } else {
        dmg -= shield;
        shield = 0;
        hp -= dmg;
        if (hp <= 0) {
            // dead
            setShieldDown();
            hp = 0;
            timer = 0;
            explosion = new Explosion;
            wep->setAmmo(0);
            model->addChild(explosion);
        }
    }
}

void Player::setWeapon(Weapon *w) {
    if (wep)delete wep;
    wep = w;
}

osg::Vec3 Player::getShootPoint(int i) {
    double rad = osg::DegreesToRadians(yrotation);
    osg::Vec3 v1;
    double x1 = weaponPoint[i].x() * cos(rad) - weaponPoint[i].y() * sin(rad);
    double y1 = weaponPoint[i].x() * sin(rad) + weaponPoint[i].y() * cos(rad);
    v1.set(x1, y1, 0);
    return v1 + getPosition();
}

bool Player::isCollisionImmune() {
    if (hp <= 0 || timer < 2000) { // immune when dead and after respawn
        return true;
    }
    return collisionImmune;
}

double Player::getCollisonRadius() {
    return shield > 0 ? collisionRadiusShield : collisionRadius;
}

void Player::shoot(Projectile*& p1, Projectile*& p2) {
    if (wep->isTwoShot()) {
        wep->shoot();
        wep->shoot();
        p1 = wep->newProjectile();
        p2 = wep->newProjectile();

        p1->setPosition(getShootPoint(1));
        p2->setPosition(getShootPoint(2));

        p1->setYRotation(getYRotation());
        p2->setYRotation(getYRotation());

        p1->setSpeed(getSpeed());
        p2->setSpeed(getSpeed());

        p1->speedUp(p1->getProjectSpeed());
        p2->speedUp(p2->getProjectSpeed());

        p1->setOwner(this);
        p2->setOwner(this);

    } else {
        wep->shoot();
        p1 = wep->newProjectile();
        p2 = NULL;

        double rad = osg::DegreesToRadians(yrotation);
        osg::Vec3 v1;
        double x1 = weaponPoint[1].x() * cos(rad) - weaponPoint[1].y() * sin(rad);
        double y1 = weaponPoint[1].x() * sin(rad) + weaponPoint[1].y() * cos(rad);
        v1.set(x1, y1, 0);

        p1->setPosition(getPosition() + v1);

        p1->setYRotation(getYRotation());

        p1->setSpeed(getSpeed());

        p1->speedUp(p1->getProjectSpeed());

        p1->setOwner(this);

    }
}

void Player::shootLaser() {
    wep->shoot();
    wep->shoot();
}

void Player::thrust(int elapsed) {
    double rad = osg::DegreesToRadians(yrotation);
    speed += osg::Vec3(cos(rad) * acceleration*elapsed, sin(rad) * acceleration *elapsed, 0);
    if (speed.length() > maxSpeed) {
        speed.normalize();
        speed *= maxSpeed;
    }
}

void Player::breaks(int elapsed) {
    speed *= pow(0.97, elapsed / 10.0);
}

void Player::steerLeft(int elapsed) {
    rotateY(steerSpeed * elapsed);
}

void Player::steerRight(int elapsed) {
    rotateY(-steerSpeed * elapsed);
}
