/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#ifndef PLAYER_H
#define PLAYER_H
class Player;
#include "gameobject.h"
#include "weapon.h"
#include "projectile.h"
#include "gamescene.h"
#include "ai.h"


#include "../gfx/enginesmoke.h"
#include "../gfx/explosion.h"
#include "../modelloader.h"
#include "../settings.h"

#include <QString>

/**
 * Class representing player
 */
class Player : public GameObject {
public:

    enum shipType {
        RAIDER = 1, WRAITH
    };
    static Player* newPlayer(int type, GameScene* gs);
    Player();

    // inherit from GameObject
    virtual void update(int elapsed);
    virtual void updatePos(int elapsed);
    bool isCollisionImmune();
    double getCollisonRadius();


    // stats

    int getScore() {
        return score;
    }

    void scoreUp(int s) {
        score += s;
    }

    int getHp() {
        return hp;
    }

    int getHp100() {
        return hp * 100 / hpMax;
    }

    int getShield() {
        return shield;
    }

    int getShield100() {
        return shieldMax == 0 ? 0 : shield * 100 / shieldMax;
    }

    int getTimer() {
        return timer;
    }

    bool isLaser() {
        return wep->type == Weapon::LASER;
    }

    QString getName() {
        return name;
    }

    bool isDead() {
        return hp == 0;
    }
    void reset();

    // shooting
    bool isWeaponReady();
    void shoot(Projectile*& p1, Projectile*& p2);
    void shootLaser();
    void damage(int dmg);

    void setName(QString n) {
        name = n;
    }

    Weapon* getWeapon() {
        return wep;
    }

    void setHp(int hp) {
        this->hp = hp;
    }

    void setHpFull() {
        this->hp = hpMax;
        this->shield = shieldMax;
    }

    void setShield(int shield) {
        this->shield = shield;
    }

    void setScore(int score) {
        this->score = score;
    }

    void setAmmo(int ammo) {
        wep->setAmmo(ammo);
    }

    void setAmmoFull() {
        wep->setAmmo(wep->getAmmoMax());
    }

    void setWeapon(Weapon* w);
    osg::Vec3 getShootPoint(int i);

    // movement
    void thrust(int elapsed);
    void breaks(int elapsed);
    void steerRight(int elapsed);
    void steerLeft(int elapsed);


    // AI
    AI* ai;
protected:

    bool ignoreControls;

    int score;
    int timer;
    QString name;

    int shield, shieldMax;
    int shieldRate;
    int shieldCD;
    int shieldCDmax;
    int hp, hpMax;

    double steerSpeed;
    double acceleration;

    Weapon* wep;
    osg::Vec3 weaponPoint[3];

    osg::Node* explosion;
    osg::ref_ptr<osg::Geode> shieldModel;
    int shieldTimer;
    bool shieldUp;
    double collisionRadiusShield;


    void setShieldUp();
    void setShieldDown();


    void createHPCircle();
    void createSPCircle();
    void createAmmoCircle();
    double hpRadius;
    osg::ref_ptr<osg::DrawArrays> circleHP;
    osg::ref_ptr<osg::DrawArrays> circleSP;
    osg::ref_ptr<osg::DrawArrays> circleAmmo;
    osg::Geometry* circleGeometryHP;
    osg::Geometry* circleGeometrySP;
    osg::Geometry* circleGeometryAmmo;
    osg::Geode* circleGeodeAmmo;
};

class PlayerWraith : public Player {
public:
    PlayerWraith(GameScene *gs);
};

class PlayerRaider : public Player {
public:
    PlayerRaider(GameScene *gs);
};

class PlayerSputnik : public Player {
public:
    PlayerSputnik();
    void update(int elapsed);
};


#endif // PLAYER_H
