/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/
#ifndef WEAPON_H
#define WEAPON_H

#include <osg/Vec3>
#include <osg/PositionAttitudeTransform>

#include "projectile.h"

/**
 * Abstract class for weapons
 */
class Weapon {
public:
    enum weaponType{CHAINGUN,LASER};
    int type;

    Weapon();

    void update(int elapsed);
    void shoot();

    bool isReady();

    bool isTwoShot() {
        return twoShot;
    }

    osg::PositionAttitudeTransform* getModel() {
        return model;
    }

    int getAmmo() {
        return ammo;
    }

    int getAmmoMax() {
        return ammoMax;
    }
    int getAmmo100(){
        return ammo*100/ammoMax;
    }

    void setAmmo(int ammo) {
        this->ammo = ammo;
    }

    virtual Projectile* newProjectile();


protected:
    osg::PositionAttitudeTransform* model;
    bool twoShot;

    int weaponCd;
    int weaponCdRemaining;

    int ammo;
    int ammoMax;
    int reloadTime;
    int reloadTimeRemaining;


};

/**
 * Level 1 gun, extends weapon
 */
class Chaingun : public Weapon {
public:
    Chaingun();
    Projectile* newProjectile();
};


/**
 * Level 2 gun, extends weapon
 */
class Laser : public Weapon {
public:
    Laser();
    Projectile* newProjectile();
};






#endif // WEAPON_H
