#include "enginesmoke.h"

EngineSmoke::EngineSmoke() {
    if (!visible)return;
    osgParticle::Particle ptemplate;

    ptemplate.setLifeTime(1);

    ptemplate.setSizeRange(osgParticle::rangef(0.75f, 2.0f));
    ptemplate.setAlphaRange(osgParticle::rangef(0.0f, 1.5f));
    ptemplate.setColorRange(osgParticle::rangev4(
            osg::Vec4(1, 0.5f, 0.3f, 1.5f),
            osg::Vec4(0, 0.7f, 1.0f, 0.0f)));

    ptemplate.setRadius(0.005f);
    ptemplate.setMass(0.00f);

    osgParticle::ParticleSystem *ps = new osgParticle::ParticleSystem;
    ps->setDefaultAttributes("data/smoke.rgb", false, false);

    ps->setDefaultParticleTemplate(ptemplate);

    // emitter
    osgParticle::ModularEmitter *emitter = new osgParticle::ModularEmitter;
    emitter->setParticleSystem(ps);

    // counter
    osgParticle::RandomRateCounter *counter = new osgParticle::RandomRateCounter;
    counter->setRateRange(60, 90);
    emitter->setCounter(counter);

    // shooter
    shooter = new osgParticle::RadialShooter;
    shooter->setInitialSpeedRange(10, 30);
    shooter->setThetaRange(0.0, 0.1);
    emitter->setShooter(shooter);

    this->addChild(emitter);
    osg::Geode *geode = new osg::Geode;
    geode->addDrawable(ps);

    this->addChild(geode);
    osgParticle::ParticleSystemUpdater *psu = new osgParticle::ParticleSystemUpdater;
    psu->addParticleSystem(ps);

    this->addChild(psu);
}

EngineSmoke2::EngineSmoke2() {
    if (!visible)return;
    osgParticle::Particle ptemplate;

    ptemplate.setLifeTime(0.5);

    ptemplate.setSizeRange(osgParticle::rangef(0.75f, 2.0f));
    ptemplate.setAlphaRange(osgParticle::rangef(0.0f, 3.0f));
    ptemplate.setColorRange(osgParticle::rangev4(
            osg::Vec4(1, 0.5f, 0.3f, 1.5f),
            osg::Vec4(0, 0.7f, 1.0f, 0.0f)));

    ptemplate.setRadius(0.01f);
    ptemplate.setMass(0.00f);

    osgParticle::ParticleSystem *ps = new osgParticle::ParticleSystem;
    ps->setDefaultAttributes("data/smoke.rgb", false, false);

    ps->setDefaultParticleTemplate(ptemplate);

    // emitter
    osgParticle::ModularEmitter *emitter = new osgParticle::ModularEmitter;
    emitter->setParticleSystem(ps);

    // counter
    osgParticle::RandomRateCounter *counter = new osgParticle::RandomRateCounter;
    counter->setRateRange(60, 90);
    emitter->setCounter(counter);

    // shooter
    shooter = new osgParticle::RadialShooter;
    shooter->setInitialSpeedRange(0, 0);
    shooter->setThetaRange(0.0, 0.1);
    emitter->setShooter(shooter);

    this->addChild(emitter);
    geode = new osg::Geode;
    geode->addDrawable(ps);
    psu = new osgParticle::ParticleSystemUpdater;
    psu->addParticleSystem(ps);
}

EngineSmoke3::EngineSmoke3() {
    if (!visible)return;
    osgParticle::Particle ptemplate;

    ptemplate.setLifeTime(0.25);

    ptemplate.setSizeRange(osgParticle::rangef(0.75f, 2.0f));
    ptemplate.setAlphaRange(osgParticle::rangef(0.0f, 3.0f));
    ptemplate.setColorRange(osgParticle::rangev4(
            osg::Vec4(0.0f, 1.0f, 0.0f, 1.5f),
            osg::Vec4(0.0f, 0.0f, 0.0f, 0.0f)));

    ptemplate.setRadius(0.01f);
    ptemplate.setMass(0.00f);

    osgParticle::ParticleSystem *ps = new osgParticle::ParticleSystem;
    ps->setDefaultAttributes("data/smoke.rgb", false, false);

    ps->setDefaultParticleTemplate(ptemplate);

    // emitter
    osgParticle::ModularEmitter *emitter = new osgParticle::ModularEmitter;
    emitter->setParticleSystem(ps);

    // counter
    osgParticle::RandomRateCounter *counter = new osgParticle::RandomRateCounter;
    counter->setRateRange(60, 90);
    emitter->setCounter(counter);

    // shooter
    shooter = new osgParticle::RadialShooter;
    shooter->setInitialSpeedRange(0, 0);
    shooter->setThetaRange(0.0, 0.1);
    emitter->setShooter(shooter);

    this->addChild(emitter);
    geode = new osg::Geode;
    geode->addDrawable(ps);
    psu = new osgParticle::ParticleSystemUpdater;
    psu->addParticleSystem(ps);
}
