/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#include "launcher.h"

Launcher* Launcher::pinstance = NULL;

Launcher::Launcher(QWidget *parent) : QMainWindow(parent) {
    pinstance = this;
    //resize(Settings::instance()->resolutionWidth, Settings::instance()->resolutionHeight);
    resize(1366, 768);
    setWindowState(windowState() | Qt::WindowMaximized);
    //fullscreen = false;

    menu = NULL;
    gcw = NULL;
    loader = NULL;
    cw = NULL;

    ignoreControls=false;

    Load();
}

Launcher::~Launcher() {

    if (menu) delete menu;
    if (gcw) delete gcw;
    if (cw) delete cw;
    if (ModelLoader::instance()) delete ModelLoader::instance();

}

Launcher* Launcher::instance() {
    return pinstance;
}

void Launcher::Menu() {
    if (menu) {
        menu->deleteLater();
    }
    if (cw) {
        cw->deleteLater();
        cw = NULL;
    }

    menu = new MenuWidget(this);

    menu->setGeometry(0, 0, width(), height());
    connect(menu->ms, SIGNAL(QuickGame()), this, SLOT(QuickGame()));
    connect(menu->ms, SIGNAL(HostGame()), this, SLOT(HostGame()));
    connect(menu->ms, SIGNAL(ClientGame()), this, SLOT(clientGame()));
    connect(menu->ms, SIGNAL(HotSeatGame()), this, SLOT(HotSeatGame()));
    connect(menu->ms, SIGNAL(restart()), this, SLOT(Restart()));
    connect(menu->ms, SIGNAL(Credits()), this, SLOT(credits()));
    menu->show();
    //loader->load();

}

void Launcher::QuickGame() {
    menu->deleteLater();
    menu = NULL;

    gcw = new GameControlWidget(HOTSEAT, this);
    connect(gcw->gc,SIGNAL(gameOver()),this,SLOT(GameOver()));
    gcw->setGeometry(0, 0, width(), height());
    gcw->show();
}

void Launcher::Quit() {
    qApp->quit();
}

void Launcher::Restart() {
    Texts::instance()->update();
    Menu();
}

void Launcher::credits() {
    menu->deleteLater();
    menu = NULL;

    cw = new CreditsWidget(this);
    cw->setGeometry(0, 0, width(), height());
    connect(cw->cs, SIGNAL(back()), this, SLOT(Menu()));
    cw->show();
}

void Launcher::GameOver(){
    if(gcw){
        gcw->deleteLater();
        gcw=NULL;
        ModelLoader::instance()->reload();
        Menu();
        menu->ms->slotGameOver();
    }
}

void Launcher::resizeEvent(QResizeEvent *event) {
    std::cout << "Launcher resizeEvent\n";
    if (menu)
        menu->resize(event->size());
    if (gcw)
        gcw->resize(event->size());
    if (cw)
        cw->resize(event->size());
}

void Launcher::keyPressEvent(QKeyEvent *evt) {
    if(ignoreControls)return;
    switch (evt->key()) {
    case Qt::Key_F9:
        switchFullScreen();
        evt->accept();
        break;
    case Qt::Key_Escape:
        escape();
        evt->accept();
        break;
    default:
        evt->ignore();
    }
}

void Launcher::switchFullScreen() {
    setWindowState(windowState() ^ Qt::WindowFullScreen);
    /*
    if (fullscreen) {
        setWindowFlags(0);
        show();
        fullscreen = false;
    } else {
        fullscreen = true;
        setWindowFlags(Qt::FramelessWindowHint);
        show();
        resize(Settings::instance()->getResolutionWidth(), Settings::instance()->getResolutionHeight());
        move(0, 0);
    }
     */
}

void Launcher::Load() {
    printf("Launcher Load\n");
    this->show();
    loader = new LoadWidget(this);
    loader->setGeometry(0, 0, width(), height());
    loader->show();
    loader->load();

    loader->hide();
    Menu();
    printf("Launcher Load end\n");
}

void Launcher::escape() {
    if (cw)
        Menu();
    if (menu)
        qApp->exit();
    if (gcw) {
        gcw->deleteLater();
        gcw = NULL;
        Menu();
        ModelLoader::instance()->reload();
    }
}

void Launcher::HotSeatGame() {
    printf("launcher hotseatgame()\n");
    menu->deleteLater();
    menu = NULL;

    gcw = new GameControlWidget(HOTSEAT, this);
    connect(gcw->gc,SIGNAL(gameOver()),this,SLOT(GameOver()));
    gcw->setGeometry(0, 0, width(), height());
    gcw->show();
}

void Launcher::HostGame() {
    printf("launcher hostgame()\n");
    menu->deleteLater();
    menu = NULL;

    gcw = new GameControlWidget(HOST, this);
    gcw->setGeometry(0, 0, width(), height());
    gcw->show();
}

void Launcher::clientGame() {
    printf("launcher client\n");
    menu->deleteLater();
    menu = NULL;

    gcw = new GameControlWidget(CLIENT, this);
    gcw->setGeometry(0, 0, width(), height());
    gcw->show();
    printf("launcher client end\n");
}
