/*****************************************************************************
 * ISP/IBP 2011/2012
 * Author: Michal Toth (xtothm00@stud.fit.vutbr.cz)
 *
 *****************************************************************************/

#include "menugamechoosermp.h"
#include "ui_menugamechoosermp.h"

MenuGameChooserMP::MenuGameChooserMP(QLabel *parent) :
    QLabel(parent),
    ui(new Ui::MenuGameChooserMP) {
    ui->setupUi(this);
    set = Settings::instance();
    ignorePackets=false;
    ttb(ui->cb1);
    ttb(ui->cb2);
    ttb(ui->cb3);

    ttb(ui->pStart);
    ttb(ui->pBack);
    ttb(ui->label);

    ttb(ui->pChatSend);

    ttb(ui->label_2);

    ports[0] = ui->p1_port;
    ports[1] = ui->p2_port;
    ports[2] = ui->p3_port;
    ports[3] = ui->p4_port;

    ui->p1_port->setPixmap(set->getPlayerIcon(1));
    ui->p2_port->setPixmap(set->getPlayerIcon(0));
    ui->p3_port->setPixmap(set->getPlayerIcon(0));
    ui->p4_port->setPixmap(set->getPlayerIcon(0));

    playerTypes[0] = 1;
    playerTypes[1] = 0;
    playerTypes[2] = 0;
    playerTypes[3] = 0;

    QVector<Map*> *mapList = Maps::instance()->getMapList();
    for (int i = 0; i < mapList->count(); i++) {
        new QListWidgetItem(mapList->at(i)->getName(), ui->mapList, 1000 + i);
    }
    ui->mapList->setCurrentRow(0);


    connect(ui->chatIn, SIGNAL(returnPressed()), this, SLOT(on_pChatSend_clicked()));
    connect(ui->p1_name, SIGNAL(textEdited(QString)), this, SLOT(sendName(QString)));
    connect(ui->p2_name, SIGNAL(textEdited(QString)), this, SLOT(sendName(QString)));
    connect(ui->p3_name, SIGNAL(textEdited(QString)), this, SLOT(sendName(QString)));
    connect(ui->p4_name, SIGNAL(textEdited(QString)), this, SLOT(sendName(QString)));
    connect(ui->mapList, SIGNAL(currentRowChanged(int)), this, SLOT(sendChooserState()));
    connect(ui->cb1, SIGNAL(toggled(bool)),this,SLOT(sendChooserState()));
    connect(ui->cb2, SIGNAL(toggled(bool)),this,SLOT(sendChooserState()));
    connect(ui->cb3, SIGNAL(toggled(bool)),this,SLOT(sendChooserState()));

    connect(ui->spinBox,SIGNAL(valueChanged(int)),this,SLOT(sendChooserState()));


    ui->label_2->setVisible(false);
    ui->spinBox->setVisible(false);
}

MenuGameChooserMP::~MenuGameChooserMP() {
    delete ui;
    if (con)delete con;
}

void MenuGameChooserMP::next(int i) {
    if (playerTypes[i] == MAX_PLAYER_TYPE) {
        playerTypes[i] = 1;
    } else {
        (playerTypes[i])++;
    }
    ports[i]->setPixmap(set->getPlayerIcon(playerTypes[i]));
    con->send(new IntPacket(Packet::MODEL_SELECT, playerTypes[i], playerNumber));
}

void MenuGameChooserMP::prev(int i) {
    if (playerTypes[i] == 1) {
        playerTypes[i] = MAX_PLAYER_TYPE;
    } else {
        playerTypes[i]--;
    }
    ports[i]->setPixmap(set->getPlayerIcon(playerTypes[i]));
    con->send(new IntPacket(Packet::MODEL_SELECT, playerTypes[i], playerNumber));
}

void MenuGameChooserMP::hosted(HostConnection* hostcon) {
    this->hostcon = hostcon;
    con = hostcon;
    host = true;
    playerNumber = 1;
    ui->p1_left->setEnabled(true);
    ui->p1_right->setEnabled(true);
    ui->p1_name->setEnabled(true);
    ui->pStart->setEnabled(true);

    ui->mapList->setEnabled(true);
    connect(hostcon, SIGNAL(packetReady()), this, SLOT(slotPacketReady()));
}

void MenuGameChooserMP::slotNewClient() {
}

void MenuGameChooserMP::joined(Connection* con) {
    this->con = con;
    host = false;
    connect(con, SIGNAL(packetReady()), this, SLOT(slotPacketReady()));
    IntPacket p(Packet::PLAYER_JOINED, 0, 0);
    con->send(&p);

    ui->cb1->setEnabled(false);
    ui->cb2->setEnabled(false);
    ui->cb3->setEnabled(false);
    ui->spinBox->setEnabled(false);

}

void MenuGameChooserMP::slotPacketReady() {
    //printf("menump slotPacketReady()\n");
    if(ignorePackets)return;
    Packet *p;

    bool started=false;

    while (!started && con && (p = con->read())) {
        switch (p->type) {
        case Packet::CHAT:
            ui->chatArea->append(((ChatPacket*) p)->msg);
            if (host)hostcon->send(p);
            break;
        case Packet::ASIGN_NUMBER:
            assignedNumber((IntPacket*) p);
            break;
        case Packet::MODEL_SELECT:
            modelSelect((IntPacket*) p);
            if (host)hostcon->send(p);
            if (host)sendChooserState();
            break;
        case Packet::CHOOSER_STATE:
            chooserState((ChooserPacket*) p);
            break;
        case Packet::PLAYER_JOINED:
            if (host)sendChooserState();
            break;
        case Packet::NAME_SELECT:
            nameSelect((ChatPacket*) p);
            if (host)sendChooserState();
            break;
        case Packet::GAME_START:
            on_pStart_clicked();
            started=true;
            break;
        case Packet::DISCONNECTED:
            packetDisconnected((IntPacket*) p);
            break;
        default:
            break;
        };

        delete p;
    }
}

void MenuGameChooserMP::nameSelect(ChatPacket *p) {
    switch (p->player) {
    case 2:
        ui->p2_name->setText(p->msg);
        break;
    case 3:
        ui->p3_name->setText(p->msg);
        break;
    case 4:
        ui->p4_name->setText(p->msg);
        break;
    }
    sendChooserState();
}

void MenuGameChooserMP::chooserState(ChooserPacket *p) {
    if (playerNumber != 1)ui->p1_name->setText(p->p1Name);
    if (playerNumber != 2)ui->p2_name->setText(p->p2Name);
    if (playerNumber != 3)ui->p3_name->setText(p->p3Name);
    if (playerNumber != 4)ui->p4_name->setText(p->p4Name);

    if (playerNumber != 1) setModel(p->p1Model, 1);
    if (playerNumber != 2) setModel(p->p2Model, 2);
    if (playerNumber != 3) setModel(p->p3Model, 3);
    if (playerNumber != 4) setModel(p->p4Model, 4);

    printf("map = %d\n", p->map);
    ui->mapList->setCurrentRow(p->map);

    ui->cb1->setChecked(p->dmg);
    ui->cb2->setChecked(p->ammo);
    ui->cb3->setChecked(p->bonus);

    ui->spinBox->setValue(p->score);
}

void MenuGameChooserMP::sendName(QString str) {
    if (host) sendChooserState();
    else {
        ChatPacket p(Packet::NAME_SELECT, str, playerNumber);
        con->send(&p);
    }
}

void MenuGameChooserMP::modelSelect(IntPacket *p) {
    setModel(p->data, p->player);
}

void MenuGameChooserMP::setModel(int m, int p) {
    int i = p - 1;
    playerTypes[i] = m;
    ports[i]->setPixmap(set->getPlayerIcon(playerTypes[i]));
}

void MenuGameChooserMP::packetDisconnected(IntPacket *p) {
    int i=p->player;
    if (!host) {
        printf("disconnect !host %d\n",p->player);
        ui->pBack->setEnabled(false);
        emit dc();
        emit leave();
    } else {
        printf("disconnect host %d\n",p->player);
        switch (p->player) {
        case 2:
            ui->p2_name->setText(Texts::get("EMPTY"));
            break;
        case 3:
            ui->p3_name->setText(Texts::get("EMPTY"));
        case 4:
            ui->p4_name->setText(Texts::get("EMPTY"));
            break;
        }

        playerTypes[i-1]=0;
        ports[i-1]->setPixmap(set->getPlayerIcon(playerTypes[i-1]));
        con->send(new IntPacket(Packet::MODEL_SELECT, playerTypes[i-1], playerNumber));
    }
}

void MenuGameChooserMP::assignedNumber(IntPacket *p) {
    playerNumber = p->player;
    QString s(tt("Player "));
    s.append(QString::number(playerNumber));
    switch (p->player) {
    case 2:
        ui->p2_left->setEnabled(true);
        ui->p2_right->setEnabled(true);
        ui->p2_name->setEnabled(true);
        ui->p2_name->setText(s);
        break;
    case 3:
        ui->p3_left->setEnabled(true);
        ui->p3_right->setEnabled(true);
        ui->p3_name->setEnabled(true);
        ui->p3_name->setText(s);
        break;
    case 4:
        ui->p4_left->setEnabled(true);
        ui->p4_right->setEnabled(true);
        ui->p4_name->setEnabled(true);
        ui->p4_name->setText(s);
        break;
    }
    next(p->data - 1);
    sendName(s);
}

void MenuGameChooserMP::sendChooserState() {
    if (host) {
        ChooserPacket *p = new ChooserPacket(ui->p1_name->text(), ui->p2_name->text(),
                                             ui->p3_name->text(), ui->p4_name->text(),
                                             playerTypes[0], playerTypes[1],
                                             playerTypes[2], playerTypes[3],
                                             ui->mapList->currentIndex().row(),
                                             ui->cb3->isChecked(),
                                             ui->cb2->isChecked(),
                                             ui->cb1->isChecked(),
                                             ui->spinBox->value()
                                             );
        hostcon->sendFast(p);
        delete p;
    }
}

void MenuGameChooserMP::on_pChatSend_clicked() {
    QString msg = ui->chatIn->text();
    msg = msg.trimmed();
    if (msg == "") {
        return;
    }
    msg.prepend(QString("[%1] ").arg(playerNumber));
    ui->chatIn->setText("");

    if (host)ui->chatArea->append(msg);
    Packet *p = new ChatPacket(msg);
    con->send(p);
    ui->chatIn->setFocus();
}

void MenuGameChooserMP::on_p1_left_clicked() {
    prev(0);
}

void MenuGameChooserMP::on_p1_right_clicked() {
    next(0);
}

void MenuGameChooserMP::on_p2_left_clicked() {
    prev(1);
}

void MenuGameChooserMP::on_p2_right_clicked() {
    next(1);
}

void MenuGameChooserMP::on_p3_left_clicked() {
    prev(2);
}

void MenuGameChooserMP::on_p3_right_clicked() {
    next(2);
}

void MenuGameChooserMP::on_p4_left_clicked() {
    prev(3);
}

void MenuGameChooserMP::on_p4_right_clicked() {
    next(3);
}

void MenuGameChooserMP::on_pBack_clicked() {
    ignorePackets=true;
    emit leave();
}

void MenuGameChooserMP::on_pStart_clicked() {
    // start
    for (int i = 0; i < MAX_PLAYERS; i++) {
        set->playerType[i]=playerTypes[i];
    }
    set->setPlayerName(0, ui->p1_name->text());
    set->setPlayerName(1, ui->p2_name->text());
    set->setPlayerName(2, ui->p3_name->text());
    set->setPlayerName(3, ui->p4_name->text());
    if (ui->mapList->selectedItems().count() != 1) return;
    set->setSelectedMap(ui->mapList->selectedItems().at(0)->type() - 1000);
    if (host) {
        Packet p(Packet::GAME_START);
        con->send(&p);
    } else {
        set->setPlayerNumber(playerNumber);
    }
    set->setCurrentConnection(con);
    con = NULL;


    set->modeDmg = ui->cb1->isChecked();
    set->modeAmmo = ui->cb2->isChecked();
    set->modeNoBonus = ui->cb3->isChecked();

    set->score=ui->spinBox->value();

    emit start();
}
