/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#include "menuscene.h"

MenuScene::MenuScene(QObject *parent) :
QOSGScene(parent) {
    std::cout << "MenuScene constructor\n";

    menuConnect = NULL;
    menugc = NULL;
    menugcmp = NULL;
    menuOptions = NULL;
    help = NULL;

    menu = new Menu();
    menu->move(50, 50);
    addWidget(menu);

    connect(menu, SIGNAL(HotSeat()), this, SLOT(slotHotSeat()));
    connect(menu, SIGNAL(QuickGame()), this, SLOT(slotQuickGame()));
    connect(menu, SIGNAL(Multiplayer()), this, SLOT(slotMultiplayer()));
    connect(menu, SIGNAL(Options()), this, SLOT(slotOptions()));
    connect(menu, SIGNAL(Credits()), this, SLOT(slotCredits()));
    connect(menu, SIGNAL(Solo()), this, SLOT(slotSolo()));

    connect(menu, SIGNAL(Help()), this, SLOT(slotHelp()));

    osg::Group* root = new osg::Group;

    //osg::Node* rot = osgDB::readNodeFile("data/raider.obj.(1.5,1.5,1.5).scale");
    osg::Node* rot = ModelLoader::instance()->modelWraith.get();
    //osg::Node* rot = osgDB::readNodeFile("data/bonusAmmo.obj.(1.5,1.5,1.5).scale");

    osg::PositionAttitudeTransform* tx = new osg::PositionAttitudeTransform;

    tx->addChild(new EngineSmoke);
    tx->setPosition(osg::Vec3(-10, 0, 1));
    tx->setAttitude(osg::Quat(osg::DegreesToRadians(90.0), osg::Vec3(0, -1, 0)));

    osg::Node* sp = ModelLoader::instance()->modelSpace.get();


    osg::PositionAttitudeTransform* scale = new osg::PositionAttitudeTransform;
    scale->addChild(sp);
    scale->addChild(rot);
    scale->setScale(osg::Vec3(1.5, 1.5, 1.5));

    root->addChild(scale);
    root->addChild(tx);


    setSceneData(root);
    orbitCamera();
    //fixedCamera();

    std::cout << "MenuScene constructor end\n";
}

MenuScene::~MenuScene() {
    std::cout << "MenuScene Destruktor\n";
    timer.stop();
    std::cout << "MenuScene Destruktor end\n";
}

void MenuScene::deleteAll() {
    if (menuConnect) {
        menuConnect->deleteLater();
        menuConnect = NULL;
    }
    if (menugc) {
        menugc->deleteLater();
        menugc = NULL;
    }
    if (menugcmp) {
        menugcmp->deleteLater();
        menugcmp = NULL;
    }
    if (menuOptions) {
        menuOptions->deleteLater();
        menuOptions = NULL;
    }
}

void MenuScene::slotHotSeat() {
    if (menugc) {
        deleteAll();
    } else {
        menugc = new MenuGameChooser();
        addWidget(menugc);
        menugc->move(50 + 310 + 50, 50);

        connect(menugc, SIGNAL(back()), this, SLOT(slotHotSeat()));
        connect(menugc, SIGNAL(start()), this, SLOT(slotQuickGame()));
    }
}

void MenuScene::slotQuickGame() {
    emit QuickGame();
}

void MenuScene::slotMultiplayer() {
    if (menuConnect || menugcmp) {
        deleteAll();
    } else {
        if (!menugcmp) {
            menuConnect = new MenuConnect;
            addWidget(menuConnect);
            menuConnect->move(50 + 310 + 50, 50);
            connect(menuConnect, SIGNAL(Host(QString, int)), this, SLOT(slotHost(QString, int)));
            connect(menuConnect, SIGNAL(Join(QString, int)), this, SLOT(slotJoin(QString, int)));
        }
    }
}

void MenuScene::slotDc() {
    QMessageBox msgBox;
    msgBox.move(width() / 2 - 100, height() / 2 - 50);
    msgBox.resize(200, 100);
    msgBox.setText(Texts::get(QString("CONNECTION_LOST")));
    msgBox.setStandardButtons(QMessageBox::Ok);
    addWidget(&msgBox);
    msgBox.exec();

}

void MenuScene::slotOptions() {
    if (menuOptions) {
        deleteAll();
    } else {
        menuOptions = new MenuOptions;
        addWidget(menuOptions);
        menuOptions->move(50 + 310 + 50, 50);

        connect(menuOptions, SIGNAL(back()), this, SLOT(slotOptions()));
        connect(menuOptions, SIGNAL(ok()), this, SLOT(slotOptionsOK()));
    }
}

void MenuScene::slotOptionsOK() {
    emit restart();
}

void MenuScene::slotHost(QString ip, int port) {
    HostConnection* hostcon = new HostConnection();
    QString *s = hostcon->init(port);
    if (s) {
        QMessageBox msgBox;
        msgBox.move(50 + 50 + 310 + 50, 150);
        msgBox.resize(200, 100);
        msgBox.setText(Texts::get(QString(*s)));
        msgBox.setStandardButtons(QMessageBox::Ok);
        addWidget(&msgBox);
        msgBox.exec();
        delete s;
        delete hostcon;
    } else {
        menuConnect->deleteLater();
        menuConnect = NULL;

        menugcmp = new MenuGameChooserMP();
        addWidget(menugcmp);
        menugcmp->move(50 + 310 + 50, 50);
        menugcmp->hosted(hostcon);

        connect(menugcmp, SIGNAL(leave()), this, SLOT(slotMultiplayer()));
        connect(menugcmp, SIGNAL(start()), this, SLOT(slotStart()));
        connect(menugcmp, SIGNAL(dc()), this, SLOT(slotDc()));

    }
}

void MenuScene::slotJoin(QString ip, int port) {
    Connection* con = new Connection();
    QString *s = con->init(ip, port);
    if (s) {
        QMessageBox msgBox;
        msgBox.move(50 + 50 + 310 + 50, 200);
        msgBox.resize(200, 100);
        msgBox.setText(Texts::get(QString(*s)));
        msgBox.setStandardButtons(QMessageBox::Ok);
        addWidget(&msgBox);
        msgBox.exec();
        delete s;
        delete con;
    } else {
        menuConnect->deleteLater();
        menuConnect = NULL;

        menugcmp = new MenuGameChooserMP();
        addWidget(menugcmp);
        menugcmp->move(50 + 310 + 50, 50);
        menugcmp->joined(con);

        connect(menugcmp, SIGNAL(leave()), this, SLOT(slotMultiplayer()));
        connect(menugcmp, SIGNAL(start()), this, SLOT(slotStart()));
        connect(menugcmp, SIGNAL(dc()), this, SLOT(slotDc()));
    }
}

void MenuScene::slotStart() {
    printf("slotstart");
    if (menugcmp->isHost())emit HostGame();
    else emit ClientGame();
}

void MenuScene::slotCredits() {
    emit Credits();
}

void MenuScene::slotSolo() {
    if (menugc) {
        deleteAll();
    } else {
        menugc = new MenuGameChooser();
        menugc->setSolo();
        addWidget(menugc);
        menugc->move(50 + 310 + 50, 50);

        connect(menugc, SIGNAL(back()), this, SLOT(slotHotSeat()));
        connect(menugc, SIGNAL(start()), this, SLOT(slotQuickGame()));
    }
}

void MenuScene::slotGameOver(){
    ScoreScreen* sc= new ScoreScreen;
    addWidget(sc);
    sc->move(50 + 310 + 50, 50);
    sc->show();

}

void MenuScene::slotHelp() {
    help = new Help;
    addWidget(help);
    help->move(50, 50);
    menu->setVisible(false);
    connect(help, SIGNAL(back()), this, SLOT(slotHelpBack()));

}

void MenuScene::slotHelpBack() {
    help->deleteLater();
    help = NULL;
    menu->setVisible(true);
}
