/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/
#ifndef CONNECTION_H
#define CONNECTION_H
class Connection;
#include <QObject>
#include <QVector>
#include <QTime>

#include <QTcpSocket>
#include <QUdpSocket>

#include "packet.h"
#include "clientinfo.h"
#include "../texts.h"


// at least one must be true
#define USE_UDP 1
#define USE_TCP 0

/**
 * Client connection, handles sending and recieving game packets
 */
class Connection : public QObject {
    Q_OBJECT
public:
    explicit Connection(QObject *parent = 0);
    virtual ~Connection();

    virtual void sendFast(Packet *p);
    virtual void send(Packet *p);

    uint getSentps() {
        return sentps;
    }

    uint getRecievedps() {
        return recievedps;
    }

    virtual Packet* read();

    QString* init(QString ip, int port);

protected:
    QVector<Packet *> packetQueue;

    ClientInfo server;

    // counter
    QTime time;
    uint sent;
    uint sentps;
    uint recieved;
    uint recievedps;


    QUdpSocket* socketUDP;
    QTcpSocket* socketTCP;

    virtual void sendTCP(QByteArray &p, QTcpSocket* soc);
    virtual void sendUDP(QByteArray &p, QUdpSocket* soc, QHostAddress addr, int port);

    virtual void readUDP(QUdpSocket*);
    virtual void readTCP(QTcpSocket*);

    void processPacket(QByteArray& packet);

signals:
    void packetReady();

public slots:
    virtual void slotTCPReady();
    virtual void slotUDPReady();

    virtual void disconnected();

};

#endif // CONNECTION_H
