/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/
#include "hostconnection.h"

HostConnection::HostConnection(QObject *parent) :
Connection(parent) {
    serverTCP = new QTcpServer(this);
}

HostConnection::~HostConnection() {
    serverTCP->close();
    delete serverTCP;
}

QString* HostConnection::init(int port) {
    if (!serverTCP->listen(QHostAddress::Any, port)) {
        return new QString("TCP_INIT_FAIL");
    }
    if (!socketUDP->bind(QHostAddress::Any, port)) {
        return new QString("UDP_INIT_FAIL");
    }
    connect(serverTCP, SIGNAL(newConnection()), this, SLOT(slotNewClient()));
    connect(socketUDP, SIGNAL(readyRead()), this, SLOT(slotUDPReady()));
    return NULL;
}

void HostConnection::sendFast(Packet *p) {
    if (USE_UDP) {

        QByteArray d = p->serialize();
        for (int i = 0; i < clients.count(); i++) {
            sendUDP(d, socketUDP, clients.at(i).addr, clients.at(i).port);
        }
    } else {
        send(p);
    }
}

void HostConnection::send(Packet *p) {
    // printf("send all TCP\n");
    if (USE_TCP) {
        QByteArray d = p->serialize();
        for (int i = 0; i < clientSocketsTCP.count(); i++) {
            sendTCP(d, clientSocketsTCP.at(i));
        }
    } else {
        sendFast(p);
    }
}

void HostConnection::slotNewClient() {
    printf("new client\n");
    while (serverTCP->hasPendingConnections()) {
        QTcpSocket *soc = serverTCP->nextPendingConnection();
        if (clientSocketsTCP.count() > 3) {
            Packet p(Packet::GAME_FULL);
            soc->write(p.serialize());
        } else {
            ClientInfo info;
            info.port = soc->peerPort();
            info.addr = soc->peerAddress();
            info.playerNubmer = getFreeNumber();
            clientSocketsTCP.append(soc);
            clients.append(info);

            IntPacket p(Packet::ASIGN_NUMBER, info.playerNubmer, info.playerNubmer);
            QByteArray d = p.serialize();
            sendTCP(d, soc);
            connect(soc, SIGNAL(readyRead()), this, SLOT(slotTCPReady()));
            connect(soc, SIGNAL(disconnected()), this, SLOT(disconnected()));
            packetQueue.append(new ChatPacket(tt("NEW_PLAYER")));
            emit packetReady();
        }
    }
}

void HostConnection::slotUDPReady() {
    readUDP(socketUDP);
    emit packetReady();
}

void HostConnection::slotTCPReady() {
    for (int i = 0; i < clientSocketsTCP.count(); i++) {
        readTCP(clientSocketsTCP.at(i));
    }
    emit packetReady();
}

int HostConnection::getFreeNumber() {
    QVector<int> n;
    n.append(2);
    n.append(3);
    n.append(4);
    for (int i = 0; i < clients.count(); i++) {
        n.remove(n.indexOf(clients.at(i).playerNubmer));
    }
    return n.first();
}

void HostConnection::readUDP(QUdpSocket *soc) {
    //printf("read udp");
    while (soc->hasPendingDatagrams()) {
        QByteArray datagram;
        datagram.resize(soc->pendingDatagramSize());
        QHostAddress sender;
        quint16 senderPort;
        soc->readDatagram(datagram.data(), datagram.size(),
                &sender, &senderPort);

        bool ok = false;
        for (int i = 0; i < clients.count(); i++) {
            if (sender == clients.at(i).addr && senderPort == clients.at(i).port)
                ok = true;
        }
        if (ok)
            processPacket(datagram);
        //else
        //printf("packet dropped\n");
    }
}

void HostConnection::disconnected() {
    //printf("client disconnected\n");
    for (int i = 0; i < clients.count(); i++) {

        QTcpSocket* s = clientSocketsTCP.at(i);
        /*
        printf("soc %d, state %d\n",i,s->state());
        printf("soc %d, isReadable %d\n",i,s->isReadable());
        printf("soc %d, isOpen %d\n",i,s->isOpen());
        printf("soc %d, isValid %d\n",i,s->isValid());
        printf("soc %d, isWritable %d\n",i,s->isWritable());*/
        if (!s->state()) {
            packetQueue.append(new IntPacket(Packet::DISCONNECTED, 42,
                                             clients.at(i).playerNubmer));
            clientSocketsTCP.remove(i);
            clients.remove(i);
            i--;
        }
    }
    emit packetReady();
}




