/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/
#include "packet.h"
#define pack(x) p.append((char*)&x,sizeof(x))
#define packLength() length=p.length(); p.prepend((char*)&length,sizeof(length));
#define unpack(x) *((x*)ptr); ptr+=sizeof(x);
#define packStr(x) p.append(x); p.append('\0');
#define unpackStr(x) QString(ptr); ptr+=x.length()+1;

Packet::Packet() {
}

Packet::Packet(uchar type) {
    this->type = type;
}

Packet::Packet(QByteArray &p) {
    char *ptr = p.data();
    length = unpack(uint)
            type = unpack(uchar);
}

QByteArray Packet::serialize() {
    QByteArray p;
    pack(type);
    packLength();
    return p;
}
//-----------------------------------------------

ChatPacket::ChatPacket(QString s) {
    type = CHAT;
    msg = s;
    player = 0;
}

ChatPacket::ChatPacket(char type, QString str, int player) {
    this->type = type;
    msg = str;
    this->player = player;
}

ChatPacket::ChatPacket(QByteArray &p) {
    char *ptr = p.data();
    length = unpack(uint);
    type = unpack(uchar);
    msg = unpackStr(msg);
    player = unpack(char);
}

QByteArray ChatPacket::serialize() {
    QByteArray p;
    pack(type);
    packStr(msg);
    pack(player);
    packLength();
    return p;
}
//-----------------------------------------------

IntPacket::IntPacket(uchar type, int data, int player) {
    this->type = type;
    this->data = data;
    this->player = player;
}

IntPacket::IntPacket(QByteArray &p) {
    char *ptr = p.data();
    length = unpack(uint);
    type = unpack(uchar);
    data = unpack(int);
    player = unpack(int);
}

QByteArray IntPacket::serialize() {
    QByteArray p;
    pack(type);
    pack(data);
    pack(player);
    packLength();
    return p;
}

//-----------------------------------------------

ChooserPacket::ChooserPacket(QString p1Name, QString p2Name,
        QString p3Name, QString p4Name,
        int p1Model, int p2Model,
        int p3Model, int p4Model,
        int map, bool bonus, bool ammo, bool dmg,
        int score) {
    type = CHOOSER_STATE;
    this->p1Name = p1Name;
    this->p2Name = p2Name;
    this->p3Name = p3Name;
    this->p4Name = p4Name;
    this->p1Model = p1Model;
    this->p2Model = p2Model;
    this->p3Model = p3Model;
    this->p4Model = p4Model;
    this->bonus = bonus;
    this->ammo = ammo;
    this->dmg = dmg;
    this->map = map;
    this->score = score;
}

ChooserPacket::ChooserPacket(QByteArray &p) {
    char *ptr = p.data();
    length = unpack(uint);
    type = unpack(uchar);
    p1Name = unpackStr(p1Name);
    p2Name = unpackStr(p2Name);
    p3Name = unpackStr(p3Name);
    p4Name = unpackStr(p4Name);
    p1Model = unpack(int);
    p2Model = unpack(int);
    p3Model = unpack(int);
    p4Model = unpack(int);
    map = unpack(int);
    bonus = unpack(bool);
    ammo = unpack(bool);
    dmg = unpack(bool);
    score = unpack(int)

}

QByteArray ChooserPacket::serialize() {
    QByteArray p;
    pack(type);
    packStr(p1Name);
    packStr(p2Name);
    packStr(p3Name);
    packStr(p4Name);
    pack(p1Model);
    pack(p2Model);
    pack(p3Model);
    pack(p4Model);
    pack(map);
    pack(bonus);
    pack(ammo);
    pack(dmg);
    pack(score);
    packLength();
    return p;
}
//-----------------------------------------------

PositionPacket::PositionPacket(int type, int x, int y, int rot, int speedx, int speedy, char player, char posType, int time) {
    this->type = type;
    this->x = x;
    this->y = y;
    this->rot = rot;
    this->speedx = speedx;
    this->speedy = speedy;
    this->time = time;
    this->player = player;
    this->posType = posType;
}

PositionPacket::PositionPacket(QByteArray &p) {
    char *ptr = p.data();
    length = unpack(uint);
    type = unpack(uchar);
    x = unpack(int);
    y = unpack(int);
    rot = unpack(int);
    speedx = unpack(int);
    speedy = unpack(int);
    posType = unpack(char);
    time = unpack(int);
    player = unpack(char);
}

QByteArray PositionPacket::serialize() {
    QByteArray p;
    pack(type);
    pack(x);
    pack(y);
    pack(rot);
    pack(speedx);
    pack(speedy);
    pack(posType);
    pack(time);
    pack(player);
    packLength();
    return p;
}
//-----------------------------------------------

CollisionPacket::CollisionPacket(char cType, int id1, int id2, int time) {
    type = COLLISION;
    this->cType = cType;
    this->id1 = id1;
    this->id2 = id2;
    this->time = time;
}

CollisionPacket::CollisionPacket(QByteArray &p) {
    char *ptr = p.data();
    length = unpack(uint);
    type = unpack(uchar);
    cType = unpack(char);
    id1 = unpack(int);
    id2 = unpack(int);
    time = unpack(int);
}

QByteArray CollisionPacket::serialize() {
    QByteArray p;
    pack(type);
    pack(cType);
    pack(id1);
    pack(id2);
    pack(time);
    packLength();
    return p;
}
//-----------------------------------------------

ScorePacket::ScorePacket(char player, int hp, int sp, int ammo, int score) {
    type = SCORE;
    this->player = player;
    this->hp = hp;
    this->sp = sp;
    this->ammo = ammo;
    this->score = score;
}

ScorePacket::ScorePacket(QByteArray &p) {
    char *ptr = p.data();
    length = unpack(uint);
    type = unpack(uchar);
    player = unpack(char);
    hp = unpack(int);
    sp = unpack(int);
    ammo = unpack(int);
    score = unpack(int);
}

QByteArray ScorePacket::serialize() {
    QByteArray p;
    pack(type);
    pack(player);
    pack(hp);
    pack(sp);
    pack(ammo);
    pack(score);
    packLength();
    return p;
}

//-----------------------------------------------

BonusPacket::BonusPacket(char bt, int x, int y) {
    type = BONUS_SPAWN;
    this->bonusType = bt;
    this->x = x;
    this->y = y;
}

BonusPacket::BonusPacket(QByteArray &p) {
    char *ptr = p.data();
    length = unpack(uint);
    type = unpack(uchar);
    bonusType = unpack(char);
    x = unpack(int);
    y = unpack(int);
}

QByteArray BonusPacket::serialize() {
    QByteArray p;
    pack(type);
    pack(bonusType);
    pack(x);
    pack(y);
    packLength();
    return p;
}
