/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/
#ifndef PACKET_H
#define PACKET_H

#define PACKET_MAX_SIZE 8192

#define D2I 1024.0 // double to int constant

#include <stdio.h>
#include <QByteArray>
#include <QString>

/**
 * Class representing simple message
 * can be serialized and sent over socket
 * base class for other backets
 */
class Packet {
public:

    enum typeEnum {
        // Packet
        GAME_FULL,
        GAME_START,

        // ChatPacket
        CHAT,
        NAME_SELECT,

        // IntPacket
        ASIGN_NUMBER,
        MODEL_SELECT,
        PLAYER_JOINED,
        DISCONNECTED,

        WANT_SHOOT,
        SHOOT,

        KILL,
        DAMAGE,

        PING,
        PING_REPLY,
        // ChooserPacket
        CHOOSER_STATE,

        // ScorePacket
        SCORE,
        // PlayerPositionPacket
        SPAWN,
        PLAYER_POSITION,
        NEW_PROJECTILE,
        // CollisionPacket
        COLLISION,

        // BonusSpawn
        BONUS_SPAWN,
        BONUS_REMOVE
    };

    enum kill {
        PLAYER, PROJECTILE
    };

    Packet();
    Packet(QByteArray &p);
    Packet(uchar type);
    virtual QByteArray serialize();

    uint length;
    uchar type;
};

/**
 * Chat packet, carries String message
 */
class ChatPacket : public Packet {
public:
    ChatPacket(QString);
    ChatPacket(char type, QString str, int player);
    ChatPacket(QByteArray &p);
    QByteArray serialize();

    QString msg;
    char player;
};

/**
 * Int packet, carries one int value
 */
class IntPacket : public Packet {
public:
    IntPacket(uchar type, int data, int player);
    IntPacket(QByteArray &p);
    QByteArray serialize();

    int data;
    int player; // or data2
};

/**
 * State of game chooser, carries all names and models from chooser and map.
 */
class ChooserPacket : public Packet {
public:
    ChooserPacket(QString p1Name, QString p2Name,
            QString p3Name, QString p4Name,
            int p1Model, int p2Model,
            int p3Model, int p4Model,
            int map,
            bool bonus, bool ammo, bool dmg,
            int score
            );
    ChooserPacket(QByteArray &p);
    QByteArray serialize();

    QString p1Name;
    QString p2Name;
    QString p3Name;
    QString p4Name;
    int p1Model;
    int p2Model;
    int p3Model;
    int p4Model;
    int map;

    bool bonus;
    bool ammo;
    bool dmg;

    int score;
};

/**
 * Position packet, carries position of one player or projectile
 */
class PositionPacket : public Packet {
public:
    PositionPacket(int type, int x, int y, int rot, int speedx, int speedy, char player, char posType, int time);
    PositionPacket(QByteArray &p);
    QByteArray serialize();

    int x, y, rot;
    int speedx, speedy;
    char player;
    char posType;
    int time;
};

/**
 * not used
 * @todo
 */
class CollisionPacket : public Packet {
public:

    enum collisionType {
        PvP, PvE, PvB, EvB
    };
    CollisionPacket(char type, int id1, int id2, int time);
    CollisionPacket(QByteArray &p);
    QByteArray serialize();

    char cType;
    int id1;
    int id2;
    int time;
};

/**
 * HP, SP, Ammo and score of one player
 */
class ScorePacket : public Packet {
public:
    ScorePacket(char player, int hp, int sp, int ammo, int score);
    ScorePacket(QByteArray &p);
    QByteArray serialize();

    char player;
    int hp;
    int sp;
    int ammo;
    int score;
};

/**
 * Data to spawn new bonus
 */
class BonusPacket : public Packet {
public:
    BonusPacket(char bt, int x, int y);
    BonusPacket(QByteArray &p);
    QByteArray serialize();

    char bonusType;
    int x;
    int y;
};
#endif // PACKET_H
