/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/
#include "qosgscene.h"

QOSGScene::QOSGScene(QObject *parent) :
QGraphicsScene(parent),
osgViewer::Viewer() {

    std::cout << "QOSGScene constructor\n";
    scene = NULL;
    m_lastStateSet = new osg::StateSet;
    setThreadingModel(osgViewer::Viewer::SingleThreaded);

    // setup camera
    cam = new osg::Camera;
    setCamera(cam);
    gwe = setUpViewerAsEmbeddedInWindow(0, 0, 1024, 768);
    setSceneRect(0, 0, 1024, 768);
    setCameraManipulator(NULL);

    std::cout << "QOSGScene constructor end\n";
}

void QOSGScene::start() {
    // auto-refreshing, used in menu
    timer.start(10);
    connect(&timer, SIGNAL(timeout()), this, SLOT(update()));
}

QOSGScene::~QOSGScene() {
    std::cout << "QOSGScene destruktor\n";
    std::cout << "QOSGScene destruktor end\n";
}

void QOSGScene::drawBackground(QPainter * painter, const QRectF & rect) {
    // render osg scene on QGLWidget background


    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glPushClientAttrib(GL_CLIENT_ALL_ATTRIB_BITS);

    // Fixup stuff that Qt Modifies from default state that
    // OSG expects.
    glShadeModel(GL_SMOOTH);

    // Push matricies
    glMatrixMode(GL_TEXTURE);
    glPushMatrix();
    glLoadIdentity();
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();

    osg::State *state = getCamera()->getGraphicsContext()->getState();
    state->reset();
    state->apply(m_lastStateSet.get());

    // rendering frame
    addRemoveModels();
    frame();

    getCamera()->getGraphicsContext()->getState()->captureCurrentState(*(m_lastStateSet.get()));

    // Pop matricies.
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_TEXTURE);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();

    // Pop Attribs, now we're ready to continue
    // Drawing with OpenGL.
    glPopAttrib();
    glPopClientAttrib();
}

void QOSGScene::resize(const QSize &size) {
    std::cout << "QOSGScene resize event\n" << size.width() << " " << size.height() << "\n";
    gwe->resized(0, 0, size.width(), size.height());
    emit resized();
}

void QOSGScene::addRemoveModels() {
    modelMut.lock();
    while (scene && !modelQueue.empty()) {
        addOrRemove a = modelQueue.first();
        modelQueue.pop_front();
        if (a.add) {
            scene->addChild(a.model);
        } else {
            scene->removeChild(a.model);
        }
    }
    modelMut.unlock();
}

void QOSGScene::addModel(osg::Node *node) {
    modelMut.lock();
    addOrRemove a = {true, node};
    modelQueue.append(a);
    modelMut.unlock();
}

void QOSGScene::removeModel(osg::Node *node) {
    modelMut.lock();
    addOrRemove a = {false, node};
    modelQueue.append(a);
    modelMut.unlock();
}

void QOSGScene::orbitCamera() {
    distance = 80;
    angle = -75;
    rotation = 0;

    timer2.start(10);
    connect(&timer2, SIGNAL(timeout()), this, SLOT(rotationUpdate()));
    time.start();
}

void QOSGScene::fixedCamera() {
    /*
    distance = 80;
    angle = -90;
    rotation = 180;

    double rad = osg::DegreesToRadians(angle);
    double radr = osg::DegreesToRadians(rotation);
     */
    osg::Vec3 up(0, 0, 1);
    // osg::Vec3 eye(sin(radr) * distance, cos(radr) * distance, distance * cos(rad));
    osg::Vec3 eye(0, -40, -60);
    osg::Vec3 center(0, 0, 0);
    getCamera()->setViewMatrixAsLookAt(eye, center, up);
}

void QOSGScene::rotationUpdate() {
    double rad = osg::DegreesToRadians(angle);
    double radr = osg::DegreesToRadians(rotation);

    osg::Vec3 up(0, 0, 1);
    osg::Vec3 eye(sin(radr) * distance, cos(radr) * distance, distance * cos(rad));
    osg::Vec3 center(0, 0, 0);
    getCamera()->setViewMatrixAsLookAt(eye, center, up);

    rotation -= 0.005 * time.elapsed();
    time.restart();
    if (rotation < 0)rotation += 360;
}
