/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/

#ifndef QOSGSCENE_H
#define QOSGSCENE_H

#include <QGraphicsScene>
#include <QTimer>
#include <QTime>
#include <QVector>
#include <QMutex>

#include <osgViewer/Viewer>
#include <osgDB/ReadFile>
#include <osgQt/GraphicsWindowQt>
#include <osgGA/TrackballManipulator>
#include <osgGA/EventQueue>
#include <osgViewer/GraphicsWindow>
#include <osg/Group>
#include <osgViewer/GraphicsWindow>

/**
 * Mutexed Add/remove queue
 * Deprecated, rewriten to single thread
 */
typedef struct {
    bool add;
    osg::Node* model;
} addOrRemove;

/**
 * Combination of OSG Viewer and QGraphicsScene to make Widget showing 3D osg scene
 */
class QOSGScene : public QGraphicsScene, public osgViewer::Viewer {
    Q_OBJECT
public:
    explicit QOSGScene(QObject *parent = 0);
    virtual ~QOSGScene();

    void drawBackground(QPainter *painter, const QRectF &rect);
    virtual void resize(const QSize &size);

    // rotates camera around center of scene, used in menu
    void orbitCamera();
    void fixedCamera();

    void addModel(osg::Node* node);
    void removeModel(osg::Node* node);
    void start();

protected:
    osg::ref_ptr<osg::Camera> cam;
    QTimer timer;
    QTime time;
    QTimer timer2;
    double rotation;
    double distance;
    double angle;


    osg::ref_ptr<osg::Group> scene;
    QVector<addOrRemove> modelQueue;
    QMutex modelMut;
    void addRemoveModels();



    osg::ref_ptr<osgViewer::GraphicsWindowEmbedded> gwe, gwe2;

    osg::ref_ptr<osg::StateSet> m_lastStateSet;

signals:
    void resized();

public slots:
    void rotationUpdate();

};

#endif // QOSGSCENE_H
