/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/
#include "settings.h"
#include <iostream>

Settings* Settings::pinstance = NULL;

Settings::Settings() {
    std::cout << "Settings constructor\n";


    // default values
    QDesktopWidget* d = QApplication::desktop();
    resolutionWidth = d->screenGeometry(d->primaryScreen()).width();
    resolutionHeight = d->screenGeometry(d->primaryScreen()).height();

    solo = true;
    modeDmg = false;
    modeNoBonus = false;
    modeAmmo = false;
    score = 20;

    lang = enGB;

    langTr.insert(enGB, QString("enGB"));
    langTr.insert(czCZ, QString("czCZ"));


    langPath = QString("data/lang/");
    confFile = QString("data/conf");


    playerName[0] = QString("Michal");
    playerName[1] = QString("Amy");
    playerName[2] = QString("Doctor Who");

    playerType[0] = 1;
    playerType[1] = 2;
    playerType[2] = 0;
    playerType[3] = 0;

    playerIcons[0] = QPixmap("data/ptNone.png");
    playerIcons[1] = QPixmap("data/ptRaider.png");
    playerIcons[2] = QPixmap("data/ptWraith.png");


    selectedMap = 0;

    vsync = false;
    particles = true;
    debug = false;
    plUi2D = false;
    plUi3D = true;

    load();
    save();

    std::cout << "Settings constructor end\n";
}

void Settings::save() {
    QFile file(confFile);
    file.open(QFile::WriteOnly);
    if (!file.isOpen())return;

    QTextStream out(&file);
    QString s;
    s = QString("vsync=%1\n").arg(vsync ? "on" : "off");
    out << s;

    s = QString("lang=%1\n").arg(langTr[lang]);
    out << s;

    s = QString("particles=%1\n").arg(particles ? "on" : "off");
    out << s;

    s = QString("plUi3D=%1\n").arg(plUi3D ? "on" : "off");
    out << s;

    s = QString("plUi2D=%1\n").arg(plUi2D ? "on" : "off");
    out << s;

    s = QString("debug=%1\n").arg(debug ? "on" : "off");
    out << s;


}

void Settings::load() {

    QFile file(confFile);
    file.open(QFile::ReadOnly);

    if (!file.isOpen())return;

    char buf[1024];
    while (file.readLine(buf, sizeof (buf)) != -1) {
        QString line(buf);
        line.replace(QString("\n"), QString(""));
        line.replace(QString("\r"), QString(""));

        if (line.length() == 0 && line.at(0) == '#') continue;

        if (line.startsWith("vsync=")) {
            if (line.section('=', 1, 1).startsWith("on")) {
                vsync = true;
            } else {
                vsync = false;
            }
        } else if (line.startsWith("lang=")) {
            if (line.section('=', 1, 1).startsWith("czCZ")) {
                lang = czCZ;
            } else if (line.section('=', 1, 1).startsWith("enGB")) {
                lang = enGB;
            }
        } else if (line.startsWith("particles=")) {
            if (line.section('=', 1, 1).startsWith("on")) {
                particles = true;
            } else {
                particles = false;
            }
        } else if (line.startsWith("plUi3D=")) {
            if (line.section('=', 1, 1).startsWith("on")) {
                plUi3D = true;
            } else {
                plUi3D = false;
            }
        } else if (line.startsWith("plUi2D=")) {
            if (line.section('=', 1, 1).startsWith("on")) {
                plUi2D = true;
            } else {
                plUi2D = false;
            }
        } else if (line.startsWith("debug=")) {
            if (line.section('=', 1, 1).startsWith("on")) {
                debug = true;
            } else {
                debug = false;
            }
        }
    }

}

Settings* Settings::instance() {
    if (pinstance == NULL) {
        pinstance = new Settings();
    }
    return pinstance;
}

void Settings::resetScores(){
    for(int i=0;i<MAX_PLAYERS;i++){
        playerScore[i]=0;
        playerName[i]=QString("");
    }
}
