/*****************************************************************************\
 *
 * Author: Michal Tth (xtothm00 _AT stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, author will kindly welcome
 * if you give him credit and keep his names with their source code
 * and inform him about any succesful project using this code.
 *
\*****************************************************************************/


#ifndef SETTINGS_H
#define SETTINGS_H
class Settings;
#define MAX_PLAYERS 4
#define MAX_PLAYER_TYPE 2
#include <QString>
#include <QPixmap>
#include <QTextOStream>
#include <QApplication>
#include <QDesktopWidget>

#include "network/connection.h"

/**
 * Class holding all game settings
 */
class Settings {
public:

    enum language {
        czCZ, enGB
    };


    static Settings* instance();

    void setCurrentConnection(Connection *p) {
        this->currentConnection = p;
    }

    Connection* getCurrentConnection() {
        return currentConnection;
    }

    QPixmap getPlayerIcon(int i) {
        return playerIcons[i];
    }

    QString getPlayerName(int i) {
        return playerName[i];
    }

    void setPlayerName(int i, QString name) {
        playerName[i] = name;
    }

    int getSelectedMap() {
        return selectedMap;
    }

    void setSelectedMap(int m) {
        selectedMap = m;
    }

    int getResolutionWidth() {
        return resolutionWidth;
    }

    int getResolutionHeight() {
        return resolutionHeight;
    }

    int getPlayerNumber() {
        return playerNumber;
    }

    void setPlayerNumber(int n) {
        playerNumber = n;
    }

    void resetScores();
public:
    int selectedMap;
    int playerType[MAX_PLAYERS];
    QString playerName[MAX_PLAYERS];
    int playerScore[MAX_PLAYERS];
    QPixmap playerIcons [MAX_PLAYER_TYPE + 1];
    QPixmap playerModels [MAX_PLAYER_TYPE + 1];



    int playerNumber;

    language lang;
    QMap<language, QString> langTr;
    bool vsync;

    bool particles;
    bool plUi3D;
    bool plUi2D;
    bool debug;

    bool solo;
    bool modeDmg;
    bool modeNoBonus;
    bool modeAmmo;
    int score;

    QString langPath;
    QString confFile;




    Connection *currentConnection;
    static Settings* pinstance;



    // not used
    int resolutionWidth;
    int resolutionHeight;
private:
    Settings();
    public 
slots:
    void save();
    void load();
};

#endif // SETTINGS_H
